#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `InvalidEdgeCitationRemover` class."""

import os
import tempfile
import shutil

import unittest
from ndex2.nice_cx_network import NiceCXNetwork
from ndexsignorloader.ndexloadsignor import InvalidEdgeCitationRemover


class TestInvalidEdgeCitationRemover(unittest.TestCase):
    """Tests for `InvalidEdgeCitationRemover` class."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_get_description(self):
        updator = InvalidEdgeCitationRemover()
        self.assertEqual('Removes any negative and non-numeric edge citations',
                         updator.get_description())

    def test_update_network_is_none(self):
        updator = InvalidEdgeCitationRemover()
        self.assertEqual(['network is None'],
                         updator.update(None))

    def test_update_network_empty(self):
        updator = InvalidEdgeCitationRemover()
        net = NiceCXNetwork()
        self.assertEqual([],
                         updator.update(net))

    def test_update_network_containing_several_citations(self):
        updator = InvalidEdgeCitationRemover()
        net = NiceCXNetwork()

        noattredge = net.create_edge(edge_source=0, edge_target=1,
                                     edge_interaction='foo')
        t_edge = net.create_edge(edge_source=2, edge_target=3,
                                 edge_interaction='blah')
        net.set_edge_attribute(t_edge,
                               InvalidEdgeCitationRemover.CITATION_ATTRIB,
                               [], type='list_of_string')

        v_edge = net.create_edge(edge_source=3, edge_target=4,
                                 edge_interaction='blah')
        net.set_edge_attribute(v_edge,
                               InvalidEdgeCitationRemover.CITATION_ATTRIB,
                               ['pubmed:1234'], type='list_of_string')

        o_edge = net.create_edge(edge_source=3, edge_target=4,
                                 edge_interaction='blah')
        net.set_edge_attribute(o_edge,
                               InvalidEdgeCitationRemover.CITATION_ATTRIB,
                               ['pubmed:Other'], type='list_of_string')

        neg_edge = net.create_edge(edge_source=5, edge_target=6,
                                   edge_interaction='blah')

        net.set_edge_attribute(neg_edge,
                               InvalidEdgeCitationRemover.CITATION_ATTRIB,
                               ['pubmed:-100', 'pubmed:5'],
                               type='list_of_string')

        issues = updator.update(net)
        self.assertEqual(2, len(issues))
        self.assertTrue('Removing invalid citation id: '
                        'pubmed:Other on edge id: ' + str(o_edge) in
                        issues)
        self.assertTrue('Removing invalid citation id: '
                        'pubmed:-100 on edge id: ' + str(neg_edge) in
                        issues)

        d_attrib = InvalidEdgeCitationRemover.CITATION_ATTRIB
        res = net.get_edge_attribute(noattredge, d_attrib)
        self.assertEqual((None, None), res)

        res = net.get_edge_attribute(t_edge, d_attrib)
        self.assertEqual([], res['v'])

        res = net.get_edge_attribute(v_edge, d_attrib)
        self.assertEqual(['pubmed:1234'], res['v'])

        res = net.get_edge_attribute(o_edge, d_attrib)
        self.assertEqual([], res['v'])

        res = net.get_edge_attribute(neg_edge, d_attrib)
        self.assertEqual(['pubmed:5'], res['v'])

    def test_update_network_with_pmc_citation(self):
        updator = InvalidEdgeCitationRemover()
        net = NiceCXNetwork()

        noattredge = net.create_edge(edge_source=0, edge_target=1,
                                     edge_interaction='foo')
        t_edge = net.create_edge(edge_source=2, edge_target=3,
                                 edge_interaction='blah')
        net.set_edge_attribute(t_edge,
                               InvalidEdgeCitationRemover.CITATION_ATTRIB,
                               ['pubmed:PMC3619734'], type='list_of_string')

        issues = updator.update(net)
        self.assertEqual(1, len(issues))
        self.assertTrue('Replacing' in issues[0])

        res = net.get_edge_attribute(t_edge,
                                     InvalidEdgeCitationRemover.CITATION_ATTRIB)
        self.assertEqual(['pubmed:15109499'], res['v'])







#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `SpringLayoutUpdator` class."""

import os
import tempfile
import shutil

import unittest
from ndex2.nice_cx_network import NiceCXNetwork
from ndexsignorloader.ndexloadsignor import SpringLayoutUpdator
from ndexsignorloader.ndexloadsignor import NodeLocationUpdator


class TestSpringLayoutUpdator(unittest.TestCase):
    """Tests for `UpdatePrefixesForNodeRepresents` class."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_get_description(self):
        updator = SpringLayoutUpdator()
        self.assertEqual('Applies Spring layout to network',
                         updator.get_description())

    def test_update_network_is_none(self):
        updator = SpringLayoutUpdator()
        self.assertEqual(['network is None'],
                         updator.update(None))

    def test_update_network_empty(self):
        updator = SpringLayoutUpdator()
        net = NiceCXNetwork()
        self.assertEqual([],
                         updator.update(net))

    def test_update_network_containing_all_types(self):
        updator = SpringLayoutUpdator()
        net = NiceCXNetwork()

        comp_attr = NodeLocationUpdator.LOCATION
        ecnode = net.create_node(node_name='ecnode', node_represents='ecnoder')
        net.set_node_attribute(ecnode, comp_attr,
                               SpringLayoutUpdator.EXTRACELLULAR)

        rnode = net.create_node(node_name='rnode', node_represents='rnoder')
        net.set_node_attribute(rnode, comp_attr,
                               SpringLayoutUpdator.RECEPTOR)

        cnode = net.create_node(node_name='cnode', node_represents='cnoder')
        net.set_node_attribute(cnode, comp_attr,
                               SpringLayoutUpdator.CYTOPLASM)

        fnode = net.create_node(node_name='fnode', node_represents='fnoder')
        net.set_node_attribute(fnode, comp_attr,
                               SpringLayoutUpdator.FACTOR)

        pnode = net.create_node(node_name='pnode', node_represents='pnoder')
        net.set_node_attribute(pnode, comp_attr,
                               SpringLayoutUpdator.PHENOTYPESLIST)

        xnode = net.create_node(node_name='xnode', node_represents='xnoder')
        self.assertEqual([], updator.update(net))

        res = net.get_opaque_aspect(SpringLayoutUpdator.CARTESIAN_LAYOUT)

        ydict = {}
        for entry in res:
            ydict[entry['node']] = entry['y']
        self.assertTrue(ydict[ecnode] < 0.0)
        self.assertTrue(ydict[rnode] < 0.0)
        self.assertTrue(ydict[cnode] > -100.0)

        self.assertTrue(ydict[fnode] > 0.0)
        self.assertTrue(ydict[pnode] > 0.0)








#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `DirectEdgeAttributeUpdator` class."""

import os
import tempfile
import shutil

import unittest
from ndex2.nice_cx_network import NiceCXNetwork
from ndexsignorloader.ndexloadsignor import DirectEdgeAttributeUpdator


class TestDirectEdgeAttributeUpdator(unittest.TestCase):
    """Tests for `DirectEdgeAttributeUpdator` class."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_get_description(self):
        updator = DirectEdgeAttributeUpdator()
        self.assertEqual('Updates value of directed edge attribute to '
                         'true and false',
                         updator.get_description())

    def test_update_network_is_none(self):
        updator = DirectEdgeAttributeUpdator()
        self.assertEqual(['network is None'],
                         updator.update(None))

    def test_update_network_empty(self):
        updator = DirectEdgeAttributeUpdator()
        net = NiceCXNetwork()
        self.assertEqual([],
                         updator.update(net))

    def test_update_network_containing_all_types(self):
        updator = DirectEdgeAttributeUpdator()
        net = NiceCXNetwork()

        noattredge = net.create_edge(edge_source=0, edge_target=1,
                                     edge_interaction='foo')
        t_edge = net.create_edge(edge_source=2, edge_target=3,
                                 edge_interaction='blah')
        net.set_edge_attribute(t_edge,
                               DirectEdgeAttributeUpdator.DIRECTED_ATTRIB,
                               't', type='string')

        f_edge = net.create_edge(edge_source=3, edge_target=4,
                                 edge_interaction='blah')
        net.set_edge_attribute(f_edge,
                               DirectEdgeAttributeUpdator.DIRECTED_ATTRIB,
                               'f', type='string')

        o_edge = net.create_edge(edge_source=3, edge_target=4,
                                 edge_interaction='blah')
        net.set_edge_attribute(o_edge,
                               DirectEdgeAttributeUpdator.DIRECTED_ATTRIB,
                               'blah', type='string')
        self.assertEqual([],
                         updator.update(net))

        d_attrib = DirectEdgeAttributeUpdator.DIRECTED_ATTRIB
        res = net.get_edge_attribute(noattredge, d_attrib)
        self.assertEqual((None, None), res)

        res = net.get_edge_attribute(t_edge, d_attrib)
        self.assertEqual(res['v'], True)

        res = net.get_edge_attribute(f_edge, d_attrib)
        self.assertEqual(res['v'], False)

        res = net.get_edge_attribute(o_edge, d_attrib)
        self.assertEqual(res['v'], False)







#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `ndexsignorloader` package."""

import os
import tempfile
import shutil

import unittest
from ndexutil.config import NDExUtilConfig
from ndexsignorloader import ndexloadsignor


class TestNdexsignorloader(unittest.TestCase):
    """Tests for `ndexsignorloader` package."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_parse_arguments(self):
        """Tests parse arguments"""
        res = ndexloadsignor._parse_arguments('hi', ['foo'])

        self.assertEqual(res.profile, 'ndexsignorloader')
        self.assertEqual(res.verbose, 0)
        self.assertEqual(res.logconf, None)
        self.assertEqual(res.conf, None)
        self.assertEqual(res.datadir, 'foo')

        someargs = ['-vv','--conf', 'foo', '--logconf', 'hi',
                    '--profile', 'myprofy', 'wellwell']
        res = ndexloadsignor._parse_arguments('hi', someargs)

        self.assertEqual(res.profile, 'myprofy')
        self.assertEqual(res.verbose, 2)
        self.assertEqual(res.logconf, 'hi')
        self.assertEqual(res.conf, 'foo')
        self.assertEqual(res.datadir, 'wellwell')

    def test_setup_logging(self):
        """ Tests logging setup"""
        try:
            ndexloadsignor._setup_logging(None)
            self.fail('Expected AttributeError')
        except AttributeError:
            pass

        # args.logconf is None
        res = ndexloadsignor._parse_arguments('hi', ['hehe'])
        ndexloadsignor._setup_logging(res)

        # args.logconf set to a file
        try:
            temp_dir = tempfile.mkdtemp()

            logfile = os.path.join(temp_dir, 'log.conf')
            with open(logfile, 'w') as f:
                f.write("""[loggers]
keys=root

[handlers]
keys=stream_handler

[formatters]
keys=formatter

[logger_root]
level=DEBUG
handlers=stream_handler

[handler_stream_handler]
class=StreamHandler
level=DEBUG
formatter=formatter
args=(sys.stderr,)

[formatter_formatter]
format=%(asctime)s %(name)-12s %(levelname)-8s %(message)s""")

            res = ndexloadsignor._parse_arguments('hi', ['--logconf',
                                                                       logfile,
                                                         'hehe'])
            ndexloadsignor._setup_logging(res)

        finally:
            shutil.rmtree(temp_dir)

    def test_main(self):
        """Tests main function"""

        # try where loading config is successful
        try:
            temp_dir = tempfile.mkdtemp()
            confile = os.path.join(temp_dir, 'some.conf')
            with open(confile, 'w') as f:
                f.write("""[hi]
                {user} = bob
                {pw} = smith
                {server} = dev.ndexbio.org""".format(user=NDExUtilConfig.USER,
                                                     pw=NDExUtilConfig.PASSWORD,
                                                     server=NDExUtilConfig.SERVER))
            res = ndexloadsignor.main(['myprog.py', '--skipdownload','--conf',
                                                     confile, '--profile',
                                                     'hi', temp_dir])
            self.assertEqual(res, 2)
        finally:
            shutil.rmtree(temp_dir)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `UpdatePrefixesForNodeRepresents` class."""

import os
import tempfile
import shutil

import unittest
from ndex2.nice_cx_network import NiceCXNetwork
from ndexsignorloader.ndexloadsignor import UpdatePrefixesForNodeRepresents


class TestUpdatePrefixesForNodeRepresents(unittest.TestCase):
    """Tests for `UpdatePrefixesForNodeRepresents` class."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_get_description(self):
        updator = UpdatePrefixesForNodeRepresents()
        self.assertEqual('Updates value of DIRECT edge attribute to yes '
                         'and no',
                         updator.get_description())

    def test_update_network_is_none(self):
        updator = UpdatePrefixesForNodeRepresents()
        self.assertEqual(['network is None'],
                         updator.update(None))

    def test_update_network_empty(self):
        updator = UpdatePrefixesForNodeRepresents()
        net = NiceCXNetwork()
        self.assertEqual([],
                         updator.update(net))

    def test_update_network_containing_all_types(self):
        updator = UpdatePrefixesForNodeRepresents()
        net = NiceCXNetwork()

        uni_one = net.create_node('somenode', node_represents='rep1')
        net.set_node_attribute(uni_one, 'DATABASE', 'UNIPROT')

        uni_two = net.create_node('somenode2',
                                  node_represents='uniprot:rep2')
        net.set_node_attribute(uni_two, 'DATABASE', 'UNIPROT')

        sig_one = net.create_node('somenode3', node_represents='rep3')
        net.set_node_attribute(sig_one, 'DATABASE', 'SIGNOR')

        sig_two = net.create_node('somenode4', node_represents='signor:rep4')
        net.set_node_attribute(sig_two, 'DATABASE', 'SIGNOR')

        other = net.create_node('somenode5',
                                node_represents='blah:rep5')
        net.set_node_attribute(other, 'DATABASE', 'other')

        self.assertEqual([],
                         updator.update(net))

        res = net.get_node(uni_one)
        self.assertEqual('uniprot:rep1', res['r'])
        self.assertEqual(None,
                         net.get_node_attribute(uni_one, 'DATABASE'))

        res = net.get_node(uni_two)
        self.assertEqual('uniprot:rep2', res['r'])
        self.assertEqual(None,
                         net.get_node_attribute(uni_two, 'DATABASE'))

        res = net.get_node(sig_one)
        self.assertEqual('signor:rep3', res['r'])
        self.assertEqual(None,
                         net.get_node_attribute(sig_one, 'DATABASE'))

        res = net.get_node(sig_two)
        self.assertEqual('signor:rep4', res['r'])
        self.assertEqual(None,
                         net.get_node_attribute(sig_two, 'DATABASE'))

        res = net.get_node(other)
        self.assertEqual('blah:rep5', res['r'])
        self.assertEqual(None,
                         net.get_node_attribute(other, 'DATABASE'))










#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `NodeMemberUpdator` class."""

import os
import tempfile
import shutil
from mock import MagicMock

import unittest
from ndex2.nice_cx_network import NiceCXNetwork
from ndexsignorloader.ndexloadsignor import NodeMemberUpdator
from ndexutil.tsv.loaderutils import GeneSymbolSearcher


class TestNodeMemberUpdator(unittest.TestCase):
    """Tests for `NodeMemberUpdator` class."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_get_description(self):
        updator = NodeMemberUpdator(None, None)
        self.assertEqual('Add genes to member node attribute for complexes '
                         'and protein families',
                         updator.get_description())

    def test_replace_signor_ids(self):
        pfdict = {'a': ['2', '3'], 'SIGNOR-PF10': ['7', '8']}
        cdict =  {'e': ['5', '6'], 'SIGNOR-C1': ['9', '10']}
        updator = NodeMemberUpdator(pfdict, cdict)
        res = updator._replace_signor_ids(['a',
                                           'SIGNOR-PF10',
                                           'SIGNOR-PF9',
                                           'e',
                                           'SIGNOR-C1',
                                           'SIGNOR-C2'])
        res[0].sort()
        self.assertEqual(['10', '7', '8', '9', 'a', 'e'], res[0])
        self.assertEqual(2, len(res[1]))
        self.assertTrue('Protein id: SIGNOR-PF9 matched prefix'
                        ' SIGNOR-PF which is assumed to be a reference to another entry, but none found. Skipping.' in res[1])
        self.assertTrue('Protein id: SIGNOR-C2 matched prefix'
                        ' SIGNOR-C which is assumed to be a reference to another entry, but none found. Skipping.' in
                        res[1])

    def test_update_network_is_none(self):
        updator = NodeMemberUpdator(None, None)
        self.assertEqual(['network is None'],
                         updator.update(None))

    def test_update_network_empty(self):
        updator = NodeMemberUpdator(None, None)
        net = NiceCXNetwork()
        self.assertEqual([],
                         updator.update(net))

    def test_add_member_genes(self):

        net = NiceCXNetwork()
        aid = net.create_node('a')
        net.set_node_attribute(aid, NodeMemberUpdator.TYPE, 'proteinfamily')

        notinid = net.create_node('x')
        net.set_node_attribute(notinid, NodeMemberUpdator.TYPE, 'proteinfamily')

        mock = GeneSymbolSearcher(bclient=None)
        mock.get_symbol = MagicMock(side_effect=['', None])
        updator = NodeMemberUpdator(None, None,
                                    genesearcher=mock)

        aidnode = net.get_node(aid)
        res = updator._add_member_genes(net, aidnode, [])
        self.assertEqual(['No proteins obtained for node: ' + str(aidnode)], res)

        notinidnode = net.get_node(notinid)
        res = updator._add_member_genes(net, notinidnode, ['x'])
        self.assertTrue('For node ' + str(notinidnode) +
                        ' No gene symbol found for x. Skipping.' in res)

        self.assertTrue('Not a single gene symbol found. Skipping insertion '
                        'of member attribute for node ' + str(notinidnode) in res)

    def test_update_network_containing_all_types(self):
        pfdict = {'a': ['2', '3'], 'SIGNOR-PF10': ['7', '8']}
        cdict = {'e': ['5', '6'], 'SIGNOR-C1': ['9', '10']}

        net = NiceCXNetwork()
        aid = net.create_node('a')
        net.set_node_attribute(aid, NodeMemberUpdator.TYPE, 'proteinfamily')

        eid = net.create_node('e')
        net.set_node_attribute(eid, NodeMemberUpdator.TYPE, 'complex')

        oid = net.create_node('c')

        o2id = net.create_node('d')
        net.set_node_attribute(o2id, NodeMemberUpdator.TYPE, 'protein')

        notinid = net.create_node('x')
        net.set_node_attribute(notinid, NodeMemberUpdator.TYPE, 'proteinfamily')

        notinid2 = net.create_node('y')
        net.set_node_attribute(notinid2, NodeMemberUpdator.TYPE, 'complex')

        mock = GeneSymbolSearcher(bclient=None)
        mock.get_symbol = MagicMock(side_effect=['AA', 'BB', 'CC', 'DD'])
        updator = NodeMemberUpdator(pfdict, cdict,
                                    genesearcher=mock)
        res = updator.update(net)
        self.assertTrue("No entry in proteinfamily map for node: {" in res[0])
        self.assertTrue("No entry in complexes map for node: {" in res[1])

        res = net.get_node_attribute(aid, NodeMemberUpdator.MEMBER)
        self.assertTrue('hgnc.symbol:AA' in res['v'])
        self.assertTrue('hgnc.symbol:BB' in res['v'])

        res = net.get_node_attribute(eid, NodeMemberUpdator.MEMBER)
        self.assertTrue('hgnc.symbol:CC' in res['v'])
        self.assertTrue('hgnc.symbol:DD' in res['v'])

        res = net.get_node_attribute(oid, NodeMemberUpdator.MEMBER)
        self.assertEqual(None, res)

        res = net.get_node_attribute(o2id, NodeMemberUpdator.MEMBER)
        self.assertEqual(None, res)



#! /usr/bin/env python

import os
import argparse
import sys
import time
import copy
import random
import requests
import logging
import csv
import json
import re
from datetime import datetime

import pandas as pd
from logging import config
import networkx

import ndex2
from ndex2.client import Ndex2
import ndexutil.tsv.tsv2nicecx2 as t2n
from ndexutil.config import NDExUtilConfig
from ndexutil.tsv.loaderutils import NetworkIssueReport
from ndexutil.tsv.loaderutils import NetworkUpdator
from ndexutil.tsv.loaderutils import GeneSymbolSearcher

import ndexsignorloader

logger = logging.getLogger(__name__)

TSV2NICECXMODULE = 'ndexutil.tsv.tsv2nicecx2'

LOG_FORMAT = "%(asctime)-15s %(levelname)s %(relativeCreated)dms " \
             "%(filename)s::%(funcName)s():%(lineno)d %(message)s"


ICON_URL = 'https://signor.uniroma2.it/img/signor_logo.png'
ICONURL_ATTRIB = '__iconurl'

NOTES_ATTRIB = 'notes'

LOAD_PLAN = 'loadplan.json'
"""
Name of file containing json load plan
stored within this package
"""

STYLE = 'style.cx'
"""
Name of file containing CX with style
stored within this package
"""

SIGNOR_URL = 'https://signor.uniroma2.it/'
"""
Base Signor URL
"""

GENERATED_BY_ATTRIB = 'prov:wasGeneratedBy'
"""
Network attribute to denote what created this network
"""

DERIVED_FROM_ATTRIB = 'prov:wasDerivedFrom'
"""
Network attribute to denote source of network data
"""

NORMALIZATIONVERSION_ATTRIB = '__normalizationversion'

SPECIES_MAPPING = {'9606': 'Human', '10090': 'Mouse', '10116': 'Rat'}


class NDExLoadSignorError(Exception):
    """
    Base exception for this module
    """
    pass


def get_package_dir():
    """
    Gets directory where package is installed
    :return:
    """
    return os.path.dirname(ndexsignorloader.__file__)


def get_load_plan():
    """
    Gets the load plan stored with this package

    :return: path to file
    :rtype: string
    """
    return os.path.join(get_package_dir(), LOAD_PLAN)


def get_style():
    """
    Gets the style stored with this package

    :return: path to file
    :rtype: string
    """
    return os.path.join(get_package_dir(), STYLE)


def _parse_arguments(desc, args):
    """
    Parses command line arguments
    :param desc:
    :param args:
    :return:
    """
    help_fm = argparse.RawDescriptionHelpFormatter
    parser = argparse.ArgumentParser(description=desc,
                                     formatter_class=help_fm)
    parser.add_argument('datadir', help='Directory where signor '
                                        'data is downloaded to '
                                        'and processed from')
    parser.add_argument('--profile', help='Profile in configuration '
                                          'file to use to load '
                                          'NDEx credentials which means'
                                          'configuration under [XXX] will be'
                                          'used '
                                          '(default '
                                          'ndexsignorloader)',
                        default='ndexsignorloader')
    parser.add_argument('--logconf', default=None,
                        help='Path to python logging configuration file in '
                             'this format: https://docs.python.org/3/library/'
                             'logging.config.html#logging-config-fileformat '
                             'Setting this overrides -v parameter which uses '
                             ' default logger. (default None)')

    parser.add_argument('--conf', help='Configuration file to load '
                                       '(default ~/' +
                                       NDExUtilConfig.CONFIG_FILE)
    parser.add_argument('--loadplan', help='Use alternate load plan file',
                        default=get_load_plan())
    parser.add_argument('--edgecollapse', action='store_true',
                        help='If set, edges with same interaction type '
                             'are collapsed and attribute values are'
                             'turned into lists except for "direct" which '
                             'is left as a bool')
    parser.add_argument('--visibility', default='PUBLIC',
                        choices=['PUBLIC', 'PRIVATE'],
                        help='Sets visibility of new '
                             'networks, default (PUBLIC)')
    parser.add_argument('--indexlevel', default='ALL',
                        choices=['NONE', 'META', 'ALL'],
                        help='Sets index level for new and updated '
                             'networks, default (ALL)')
    parser.add_argument('--disableshowcase', action='store_true',
                        help='If set, networks will NOT be showcased')
    parser.add_argument('--style',
                        help='Path to NDEx CX file to use for styling'
                             'networks, or NDEx UUID which must be '
                             'accessible and on same server as'
                             'those specified by the credentials',
                        default=get_style())
    parser.add_argument('--iconurl',
                        help='Sets network attribute value __iconurl '
                             '(default ' + ICON_URL + ')',
                        default=ICON_URL)
    parser.add_argument('--skipdownload', action='store_true',
                        help='If set, skips download of data from signor'
                             'and assumes data already resides in <datadir>'
                             'directory')
    parser.add_argument('--skipfull', action='store_true',
                        help='If set, skips generation (not download) of '
                             'Complete Human, Rat, Mouse '
                             'pathways')
    parser.add_argument('--signorurl', default=SIGNOR_URL,
                        help='URL to signor pathways (default ' +
                        SIGNOR_URL + ')')
    parser.add_argument('--verbose', '-v', action='count', default=0,
                        help='Increases verbosity of logger to standard '
                             'error for log messages in this module and'
                             'in ' + TSV2NICECXMODULE + '. Messages are '
                             'output at these python logging levels '
                             '-v = ERROR, -vv = WARNING, -vvv = INFO, '
                             '-vvvv = DEBUG, -vvvvv = NOTSET (default no '
                             'logging)')
    parser.add_argument('--version', action='version',
                        version=('%(prog)s ' +
                                 ndexsignorloader.__version__))

    return parser.parse_args(args)


def _setup_logging(args):
    """
    Sets up logging based on parsed command line arguments.
    If args.logconf is set use that configuration otherwise look
    at args.verbose and set logging for this module and the one
    in ndexutil specified by TSV2NICECXMODULE constant
    :param args: parsed command line arguments from argparse
    :raises AttributeError: If args is None or args.logconf is None
    :return: None
    """

    if args.logconf is None:
        level = (50 - (10 * args.verbose))
        logging.basicConfig(format=LOG_FORMAT,
                            level=level)
        logging.getLogger(TSV2NICECXMODULE).setLevel(level)
        logger.setLevel(level)
        return

    # logconf was set use that file
    logging.config.fileConfig(args.logconf,
                              disable_existing_loggers=False)


class SignorDownloader(object):
    """
    Downloads signor data from site
    """
    PATHWAYDATA_SCRIPT = 'getPathwayData.php?list'

    PATHWAY_LIST_FILE = 'pathway_list.txt'
    PATHWAYDATA_DOWNLOAD_SCRIPT = 'getPathwayData.php?pathway='
    GETDATA_SCRIPT = 'getData.php?organism='
    DOWNLOAD_COMPLEXES = 'download_complexes.php'
    PROTEINFAMILY_FILE = 'SIGNOR_PF.csv'
    COMPLEXES_FILE = 'SIGNOR_complexes.csv'

    def __init__(self, signorurl, outdir):
        """
        Constructor

        :param signorurl: base SIGNOR URL
        :type signorurl: string
        :param outdir: directory where files will be downloaded to
        :type outdir: string
        """
        self._signorurl = signorurl
        self._outdir = outdir

    def get_pathway_list_file(self):
        """
        gets pathway list file
        :return:
        """
        return os.path.join(self._outdir,
                            SignorDownloader.PATHWAY_LIST_FILE)

    def get_proteinfamily_file(self):
        """
        Gets complexes file containing mapping of protein families to genes

        :return:
        """
        return os.path.join(self._outdir,
                            SignorDownloader.PROTEINFAMILY_FILE)

    def get_complexes_file(self):
        """
        Gets complexes file containing mapping of protein families to genes

        :return:
        """
        return os.path.join(self._outdir,
                            SignorDownloader.COMPLEXES_FILE)

    def _get_entity_file_map(self, entityfile):
        """
        Given a path to a semi-colon delimited file which is assumed
        to be either a SIGNOR proteinfamily or SIGNOR complexes file
        create a dictionary with key set to contents of row in second column
        and values set to comma split and trimmed list in 3rd column

        :param entityfile:
        :return:
        """
        path_map = {}
        with open(entityfile, 'r') as f:
            reader = csv.reader(f, delimiter=';')
            for line in reader:
                idlist = []
                for entry in line[2].split(','):
                    idlist.append(entry.strip())
                path_map[line[1]] = idlist
                path_map[line[0]] = idlist
        return path_map

    def get_proteinfamily_map(self):
        """

        :return:
        """
        return self._get_entity_file_map(self.get_proteinfamily_file())

    def get_complexes_map(self):
        """

        :return:
        """
        return self._get_entity_file_map(self.get_complexes_file())

    def _download_entity_file(self, entity_data_type, destfile):
        """
        Downloads entity file

        :param entity_data_type: entity data type
        :type entity_data_type: str
        :param destfile: path where entity file should be downloaded to
        :type destfile: str
        :return: None
        """
        logger.info('Downloading ' + entity_data_type)
        postdata = {'Content-Disposition': 'form-data; name="submit"',
                    'submit': entity_data_type}
        resp = requests.post(self._signorurl + '/' +
                             SignorDownloader.DOWNLOAD_COMPLEXES,
                             data=postdata)
        if resp.status_code != 200:
            raise NDExLoadSignorError('Got status code of ' +
                                      str(resp.status_code) +
                                      ' from signor')
        with open(destfile, 'w') as f:
            f.write(resp.text)
            f.flush()

    def _download_pathways_list(self):
        """
        Gets map of pathways

        :return: dict
        """
        logger.info("Downloading pathways list")
        resp = requests.get(self._signorurl + '/' +
                            SignorDownloader.PATHWAYDATA_SCRIPT)
        if resp.status_code != 200:
            raise NDExLoadSignorError('Got status code of ' +
                                      str(resp.status_code) + ' from signor')
        with open(self.get_pathway_list_file(), 'w') as f:
            f.write(resp.text)
            f.flush()

    def get_pathways_map(self):
        """
        Gets map from :py:const:`SignorDownloader.PATHWAY_LIST_FILE` file

        :return:
        """
        path_map = {}
        with open(self.get_pathway_list_file(), 'r') as f:
            reader = csv.reader(f, delimiter='\t')
            for line in reader:
                path_map[line[0].replace('/', '')] = line[1]

        return path_map

    def _get_download_url(self, pathway_id, relationsonly=False):
        """
        Builds download URL

        :param pathway_id: id of pathway
        :type pathway_id: str
        :param relationsonly: if True appends &relations=only to URL
        :type relationsonly: bool
        :return: download URL
        :rtype: str
        """
        if relationsonly is False:
            relationssuffix = ''
        else:
            relationssuffix = '&relations=only'

        return self._signorurl + '/' +\
               SignorDownloader.PATHWAYDATA_DOWNLOAD_SCRIPT +\
               pathway_id + relationssuffix

    def _download_file(self, download_url, destfile):
        """
        Downloads file pointed to by 'download_url' to
        'destfile'

        :param theurl: link to download
        :type theurl: str
        :param destfile: path to file to write contents of 'theurl' to
        :type destfile: str
        :return: None
        """
        if os.path.isfile(destfile):
            logger.info(destfile + ' exists. Skipping...')
            return

        logger.info('Downloading ' + download_url + ' to ' + destfile)
        with requests.get(download_url, stream=True) as r:
            r.raise_for_status()
            with open(destfile, 'wb') as f:
                for chunk in r.iter_content(chunk_size=8192):
                    if chunk:  # filter out keep-alive new chunks
                        f.write(chunk)

    def _download_pathway(self, pathway_id, destfile, relationsonly=False):
        """

        :param pathway_id:
        :param destfile:
        :param relationsonly:
        :return:
        """
        download_url = self._get_download_url(pathway_id,
                                              relationsonly=relationsonly)
        self._download_file(download_url, destfile)

    def _get_fulldownload_url(self, species_id):
        """

        :param species_id:
        :return:
        """
        return self._signorurl + '/' +\
               SignorDownloader.GETDATA_SCRIPT + species_id

    def _download_fullspecies(self, species_id, destfile):
        """

        :param species_id:
        :param destfile:
        :return:
        """
        download_url = self._get_fulldownload_url(species_id)
        self._download_file(download_url, destfile)

    def download_data(self):
        """
        Downloads data
        :return:
        """
        if not os.path.isdir(self._outdir):
            os.makedirs(self._outdir, mode=0o755)

        self._download_pathways_list()
        self._download_entity_file('Download protein family data',
                                   self.get_proteinfamily_file())

        self._download_entity_file('Download complex data',
                                   self.get_complexes_file())

        path_map = self.get_pathways_map()
        for key in path_map.keys():
            self._download_pathway(key, os.path.join(self._outdir,
                                                     key + '.txt'),
                                   relationsonly=True)
            self._download_pathway(key, os.path.join(self._outdir,
                                                     key + '_desc.txt'),
                                   relationsonly=False)

        for key in SPECIES_MAPPING.keys():
            self._download_fullspecies(key,
                                       os.path.join(self._outdir,
                                                    'full_' +
                                                    SPECIES_MAPPING[key] +
                                                    '.txt'))


class DirectEdgeAttributeUpdator(NetworkUpdator):
    """
    Updates value of
    :py:const:`DirectEdgeAttributeUpdator.DIRECTED_ATTRIB`
    edge attribute

    """

    DIRECTED_ATTRIB = 'direct'

    def __init__(self):
        """
        Constructor

        """
        super(DirectEdgeAttributeUpdator, self).__init__()

    def get_description(self):
        """

        :return:
        """
        return 'Updates value of directed edge attribute to true and false'

    def update(self, network):
        """
        Iterates through all edges in network updating edge attribute
        :py:const:`DirectEdgeAttributeUpdator.DIRECTED_ATTRIB` to 'True'
        if value is 't' otherwise set it to 'False'

        :param network: network to examine
        :type network: :py:class:`~ndex2.nice_cx_network.NiceCXNetwork`
        :return: empty list
        :rtype: list
        """
        if network is None:
            return ['network is None']

        issues = []
        directed_attr_name = DirectEdgeAttributeUpdator.DIRECTED_ATTRIB
        for edge_id, edge in network.get_edges():
            edge_attr = network.get_edge_attribute(edge_id,
                                                    directed_attr_name)
            if edge_attr == (None, None):
                continue
            network.remove_edge_attribute(edge_id, directed_attr_name)
            if edge_attr['v'] == 't':

                network.set_edge_attribute(edge_id,
                                           directed_attr_name, True,
                                           type='boolean')
            else:
                network.set_edge_attribute(edge_id,
                                           directed_attr_name, False,
                                           type='boolean')

        return issues


class InvalidEdgeCitationRemover(NetworkUpdator):
    """
    Looks at citation edge attribute and removes any
    non-numeric values leaving an empty list
    """

    CITATION_ATTRIB = 'citation'

    PMC_MAP = {'PMC3619734': '15109499'}
    """
    There is one PMCID entry seen in the full networks.
    For now the above map replaces that PMC id with a 
    pubmed identifier, but in future might need to 
    setup a service to query for these from:
    
    https://www.ncbi.nlm.nih.gov/pmc/utils/idconv/v1.0/?tool=ndexsignorloader&email=my_email@example.com&ids=PMC3619734
    """

    def __init__(self):
        """
        Constructor

        """
        super(InvalidEdgeCitationRemover, self).__init__()

    def get_description(self):
        """

        :return:
        """
        return 'Removes any negative and non-numeric edge citations'

    def update(self, network):
        """
        Iterates through all edges in network removing any non-numeric
        or negative citations after pubmed: prefix is removed.

        :param network: network to examine
        :type network: :py:class:`~ndex2.nice_cx_network.NiceCXNetwork`
        :return: empty list
        :rtype: list
        """
        if network is None:
            return ['network is None']

        pmc_map = InvalidEdgeCitationRemover.PMC_MAP
        issues = []
        citation_attr_name = InvalidEdgeCitationRemover.CITATION_ATTRIB
        for edge_id, edge in network.get_edges():
            edge_attr = network.get_edge_attribute(edge_id,
                                                   citation_attr_name)
            if edge_attr == (None, None):
                continue

            update_citation = False
            updatedcitations = []
            for entry in edge_attr['v']:
                idonly = re.sub('^pubmed:', '', entry)
                # logger.info('Entry => ' +  entry + ' id: ' + idonly)
                if idonly.isdigit():
                    updatedcitations.append(entry)
                elif idonly in pmc_map:
                    update_citation = True
                    updatedcitations.append('pubmed:' +
                                            pmc_map[idonly])
                    issues.append('Replacing ' + idonly +
                                  ' with pubmed id: ' + pmc_map[idonly] +
                                  ' on edge id: ' + str(edge_id))
                else:
                    update_citation = True
                    issues.append('Removing invalid citation id: ' +
                                  str(entry) + ' on edge id: ' + str(edge_id))

            if update_citation is True:
                network.remove_edge_attribute(edge_id, citation_attr_name)
                network.set_edge_attribute(edge_id,
                                           citation_attr_name,
                                           updatedcitations,
                                           type='list_of_string')

        return issues


class UpdatePrefixesForNodeRepresents(NetworkUpdator):
    """
    Prefixes node represents with uniprot: or signor:
    based on value of :py:const:`UpdatePrefixesForNodeRepresents.DATABASE`
    node attribute
    """

    DATABASE = 'DATABASE'

    def __init__(self):
        """
        Constructor

        """
        super(UpdatePrefixesForNodeRepresents, self).__init__()

    def get_description(self):
        """

        :return:
        """
        return 'Updates value of DIRECT edge attribute to yes and no'

    def update(self, network):
        """
        Iterates through nodes and updates prefix for represents of
        node based on value of
        :py:const:`UpdatePrefixesForNodeRepresents.DATABASE` node
        attribute. If the value if 'UNIPROT' then 'uniprot:' is
        prefixed if not already there
        and if 'SIGNOR' then 'signor:' is prefixed.
        The :py:const:`UpdatePrefixesForNodeRepresents.DATABASE` is
        removed as well

        :param network: network to examine
        :type network: :py:class:`~ndex2.nice_cx_network.NiceCXNetwork`
        :return: list of node names for which no replacement was found
        :rtype: list
        """
        if network is None:
            return ['network is None']

        issues = []
        db_attribute = UpdatePrefixesForNodeRepresents.DATABASE

        for node_id, node in network.get_nodes():
            database = network.get_node_attribute_value(node_id,
                                                        db_attribute)
            represents = node.get('r')
            if database == "UNIPROT":
                if 'uniprot:' not in represents:
                    represents = "uniprot:" + represents
                    node['r'] = represents
            elif database == "SIGNOR":
                if 'signor:' not in represents:
                    represents = "signor:" + represents
                    node['r'] = represents
            # in all other cases, the identifier is already prefixed
            network.remove_node_attribute(node_id,
                                          db_attribute)

        return issues


class NodeLocationUpdator(NetworkUpdator):
    """
    Replace any empty node Location attribute values with cytoplasm and
    replace andy phenotypesList Location attribute values with empty
    string
    """
    CYTOPLASM = 'cytoplasm'
    PHENOTYPESLIST = 'phenotypesList'
    LOCATION = 'location'

    def __init__(self):
        """
        Constructor

        """
        super(NodeLocationUpdator, self).__init__()

    def get_description(self):
        """

        :return:
        """
        return 'Replace any empty node location attribute values with ' \
               'cytoplasm'

    def update(self, network):
        """
        Iterates through nodes and updates 'location' attribute
        if its empty it to 'cytoplasm' or if its 'phenotypesList' set
        it to empty string

        :param network: network to examine
        :type network: :py:class:`~ndex2.nice_cx_network.NiceCXNetwork`
        :return: list of node names for which no replacement was found
        :rtype: list
        """
        if network is None:
            return ['network is None']

        issues = []
        comp_attr = NodeLocationUpdator.LOCATION
        for node_id, node in network.get_nodes():
            node_attr = network.get_node_attribute(node_id,
                                                   comp_attr)
            if node_attr == (None, None) or node_attr is None:
                logger.debug('Node ' + str(node_id) +
                             ' did not have ' + comp_attr +
                             ' attribute. Setting to ' +
                             NodeLocationUpdator.CYTOPLASM)

                network.set_node_attribute(node_id, comp_attr,
                                           NodeLocationUpdator.CYTOPLASM)
                continue
            if node_attr['v'] is None or node_attr['v'] == '':
                node_attr['v'] = NodeLocationUpdator.CYTOPLASM
            elif node_attr['v'] == NodeLocationUpdator.PHENOTYPESLIST:
                node_attr['v'] = ''
        return issues


class SpringLayoutUpdator(NetworkUpdator):
    """
    Applies Spring layout
    """
    CARTESIAN_LAYOUT = 'cartesianLayout'

    PHENOTYPESLIST = ''
    FACTOR = 'factor'
    CYTOPLASM = 'cytoplasm'
    RECEPTOR = 'receptor'
    EXTRACELLULAR = 'extracellular'

    def __init__(self, scale=500.0,
                 iterations=10, seed=10,
                 location_weight=5.0):
        """
        Constructor

        :param scale: scale for networkx spring layout
        :type scale: float
        """
        super(SpringLayoutUpdator, self).__init__()

        self._scale = scale
        self._seed = seed
        self._iterations = iterations
        self._min = -scale
        self._max = scale
        self._location_weight = location_weight
        random.seed(self._seed)

    def get_description(self):
        """

        :return:
        """
        return 'Applies Spring layout to network'

    def _get_random_x_position(self):
        """

        :return:
        """
        return random.uniform(self._min, self._max)

    def _are_all_nodes_same_location(self, network):
        """

        :param network:
        :return:
        """
        node_loc = set()
        compartment = NodeLocationUpdator.LOCATION
        for nodeid, node in network.get_nodes():
            node_attr = network.get_node_attribute(nodeid,
                                                   compartment)
            if node_attr is None:
                continue
            node_loc.add(node_attr['v'])
        if len(node_loc) <= 1:
            return True
        return False

    def _get_initial_node_positions(self, network):
        """
        Based on Compartment node attribute position nodes

        :param network:
        :return:
        """
        node_pos = {}
        compartment = NodeLocationUpdator.LOCATION
        for nodeid, node in network.get_nodes():
            node_attr = network.get_node_attribute(nodeid,
                                                   compartment)
            if node_attr is None:
                continue
            if node_attr['v'] == SpringLayoutUpdator.EXTRACELLULAR:
                node_pos[nodeid] = (self._get_random_x_position(),
                                    self._min)
            elif node_attr['v'] == SpringLayoutUpdator.RECEPTOR:
                node_pos[nodeid] = (self._get_random_x_position(),
                                    self._min/2.0)
            elif node_attr['v'] == SpringLayoutUpdator.CYTOPLASM:
                node_pos[nodeid] = (self._get_random_x_position(), 0.0)
            elif node_attr['v'] == SpringLayoutUpdator.FACTOR:
                node_pos[nodeid] = (self._get_random_x_position(),
                                    self._max/2.0)
            elif node_attr['v'] == '':
                node_pos[nodeid] = (self._get_random_x_position(),
                                    self._max)

        node_pos[SpringLayoutUpdator.EXTRACELLULAR] = (0.0, self._min)
        node_pos[SpringLayoutUpdator.RECEPTOR] = (0.0, self._min/2.0)
        node_pos[SpringLayoutUpdator.CYTOPLASM] = (0.0, 0.0)
        node_pos[SpringLayoutUpdator.FACTOR] = (0.0, self._max/2.0)
        node_pos[SpringLayoutUpdator.PHENOTYPESLIST] = (0.0, self._max)

        return node_pos

    def _get_cartesian_aspect(self, net_x):
        """
        Converts coordinates from :py:class:`networkx.Graph`
        to NDEx aspect
        :param net_x: network with coordinates
        :type net_x: :py:class:`networkx.Graph`
        :return: coordinates as list of dicts ie
                 [{'node': <id>, 'x': <xpos>, 'y': <ypos>}]
        :rtype: list
        """
        # return [{'node': n,
        #         'x': float(net_x.pos[n][0]),
        #         'y': float(net_x.pos[n][1])} for n in net_x.pos]
        coords = []
        for n in net_x.pos:
            if not isinstance(n, int):
                continue
            coords.append({'node': n,
                           'x': float(net_x.pos[n][0]),
                           'y': float(net_x.pos[n][1])})
        return coords

    def _get_networkx_object(self, network, nodes_same_loc=False):
        """

        :param network:
        :return:
        """
        net_x = network.to_networkx(mode='default')
        if nodes_same_loc is True:
            return net_x
        net_x.add_node(SpringLayoutUpdator.EXTRACELLULAR,
                       weight=self._location_weight)
        net_x.add_node(SpringLayoutUpdator.RECEPTOR,
                       weight=self._location_weight)
        net_x.add_node(SpringLayoutUpdator.CYTOPLASM,
                       weight=self._location_weight)
        net_x.add_node(SpringLayoutUpdator.FACTOR,
                       weight=self._location_weight)
        net_x.add_node(SpringLayoutUpdator.PHENOTYPESLIST,
                       weight=self._location_weight)

        compartment = NodeLocationUpdator.LOCATION
        for nodeid, node in network.get_nodes():
            node_attr = network.get_node_attribute(nodeid,
                                                   compartment)
            if node_attr is None:
                continue
            if node_attr['v'] is not None:
                net_x.add_edge(nodeid, node_attr['v'])
        return net_x

    def update(self, network):
        """
        Applies spring layout to network

        :param network: network to examine
        :type network: :py:class:`~ndex2.nice_cx_network.NiceCXNetwork`
        :return: list of node names for which no replacement was found
        :rtype: list
        """
        if network is None:
            return ['network is None']

        issues = []
        nodes_same_loc = self._are_all_nodes_same_location(network)

        net_x = self._get_networkx_object(network, nodes_same_loc=nodes_same_loc)

        numnodes = len(network.get_nodes())
        updatedscale = self._scale - numnodes
        updatedk = 1000.0 + numnodes*20
        pos_dict = self._get_initial_node_positions(network)

        # reason for this is if all nodes are of same type
        # the nodes end up all on the same y coordinate in a big
        # row. Partial fix for issue:
        # https://ndexbio.atlassian.net/browse/UD-1677
        if nodes_same_loc is True:
            net_x.pos = networkx.drawing.spring_layout(net_x, scale=updatedscale,
                                                       seed=self._seed,
                                                       k=updatedk,
                                                       iterations=self._iterations)
        else:
            net_x.pos = networkx.drawing.spring_layout(net_x, scale=updatedscale,
                                                       seed=self._seed,
                                                       pos=pos_dict,
                                                       k=updatedk,
                                                       iterations=self._iterations)

        network.set_opaque_aspect(SpringLayoutUpdator.CARTESIAN_LAYOUT,
                                  self._get_cartesian_aspect(net_x))
        return issues


class NodeMemberUpdator(NetworkUpdator):
    """
    Adds genes to 'member' attribute for any nodes that
    are of type 'complex', 'proteinfamily'
    """
    TYPE = 'type'
    MEMBER = 'member'
    PROTEINFAMILY = 'proteinfamily'
    COMPLEX = 'complex'
    SIGNOR_PF_PREFIX = 'SIGNOR-PF'
    SIGNOR_C_PREFIX = 'SIGNOR-C'

    def __init__(self, proteinfamily_map, complexes_map,
                 genesearcher=GeneSymbolSearcher()):
        """
        Constructor

        """
        super(NodeMemberUpdator, self).__init__()
        self._proteinfamilymap = proteinfamily_map
        self._complexesmap = complexes_map
        self._genesearcher = genesearcher

    def get_description(self):
        """

        :return:
        """
        return 'Add genes to member node attribute for complexes and protein ' \
               'families'

    def _add_member_genes(self, network, node, proteinlist):
        """

        :param network:
        :param node:
        :param proteinlist:
        :return:
        """

        if proteinlist is None or len(proteinlist) == 0:
            return ['No proteins obtained for node: ' + str(node)]

        issues = []
        member_set = set()
        for entry in proteinlist:
            g_symbol = self._genesearcher.get_symbol(entry)
            if g_symbol is None or g_symbol == '':
                issues.append('For node ' + str(node) +
                              ' No gene symbol found for ' +
                              str(entry) + '. Skipping.')
                continue

            symbol_w_prefix = 'hgnc.symbol:' + g_symbol

            # this is merely to report the duplicate cause
            # the set will automatically skip the insertion
            # fix for UD-2078
            if symbol_w_prefix in member_set:
                issues.append(entry + ' mapping to gene: ' + g_symbol +
                              ' already in list. Skipping insertion')

            member_set.add(symbol_w_prefix)
        if len(member_set) == 0:
            issues.append('Not a single gene symbol found. Skipping '
                          'insertion of member attribute for node ' +
                          str(node))
            return issues

        network.set_node_attribute(node['@id'], NodeMemberUpdator.MEMBER,
                                   list(member_set),
                                   type='list_of_string', overwrite=True)
        return issues

    def _replace_signor_ids(self, proteinlist):
        """
        Iterate through 'proteinlist` and if any entries start
        with SIGNOR-PF or SIGNOR-C do another lookup in proteinfamilymap
        or complexesmap set in constructor for the genes. If found add
        these genes to the list.
        :param proteinlist:
        :return:
        """
        updatedlist = []
        issues = []
        for entry in proteinlist:
            if entry.startswith(NodeMemberUpdator.SIGNOR_PF_PREFIX):
                if entry not in self._proteinfamilymap:
                    issues.append('Protein id: ' + entry +
                                  ' matched prefix ' +
                                  NodeMemberUpdator.SIGNOR_PF_PREFIX +
                                  ' which is assumed to be a reference '
                                  'to another'
                                  ' entry, but none found. Skipping.')
                    continue
                updatedlist.extend(self._proteinfamilymap[entry])
                continue
            elif entry.startswith(NodeMemberUpdator.SIGNOR_C_PREFIX):
                if entry not in self._complexesmap:
                    issues.append('Protein id: ' + entry + ' matched prefix ' +
                                  NodeMemberUpdator.SIGNOR_C_PREFIX +
                                  ' which is assumed to be a '
                                  'reference to another'
                                  ' entry, but none found. Skipping.')
                    continue
                updatedlist.extend(self._complexesmap[entry])
                continue
            updatedlist.append(entry)
        return list(set(updatedlist)), issues

    def update(self, network):
        """
        Iterates through nodes and updates 'member' attribute
        for nodes with 'type' attribute set to 'proteinfamily' or 'complex'
        by getting mapping from signor

        :param network: network to examine
        :type network: :py:class:`~ndex2.nice_cx_network.NiceCXNetwork`
        :return: list of node names for which no replacement was found
        :rtype: list
        """
        if network is None:
            return ['network is None']

        issues = []
        type_attr = NodeMemberUpdator.TYPE
        for node_id, node in network.get_nodes():
            node_attr = network.get_node_attribute(node_id,
                                                   type_attr)
            if node_attr == (None, None) or node_attr is None:
                logger.debug('Node ' + str(node_id) +
                             ' did not have ' + type_attr +
                             ' attribute.')
                continue
            if node_attr['v'] == NodeMemberUpdator.PROTEINFAMILY:
                if node['n'] not in self._proteinfamilymap:
                    logger.error('Node: ' + node['n'] +
                                 ' not in proteinfamily map')
                    issues.append('No entry in proteinfamily map for node: ' +
                                  str(node))
                    continue
                proteinlist, oissues = self._replace_signor_ids(self._proteinfamilymap[node['n']])
                issues.extend(oissues)
                issues.extend(self._add_member_genes(network, node,
                                                     proteinlist))
            elif node_attr['v'] == NodeMemberUpdator.COMPLEX:
                if node['n'] not in self._complexesmap:
                    logger.error('Node: ' + node['n'] + ' not in complexes map')
                    issues.append('No entry in complexes map for node: ' +
                                  str(node))
                    continue
                proteinlist, oissues = self._replace_signor_ids(self._complexesmap[node['n']])
                issues.extend(oissues)
                issues.extend(self._add_member_genes(network, node,
                                                     proteinlist))
        return issues


class RedundantEdgeCollapser(NetworkUpdator):
    """
    Examines network and collapses edges as
    described in :py:func:`~RedundantEdgeCollapser.update`
    """
    SENTENCE = 'sentence'
    CITATION = 'citation'

    def __init__(self):
        """
        Constructor

        """
        super(RedundantEdgeCollapser, self).__init__()
        self._pubmedurl = None

    def get_description(self):
        """
        Gets description of this network updator
        :return:
        """
        return 'Collapses redundant edges'

    def _remove_edge(self, network, edgeid):
        """
        Removes edge and its attributes

        :param network: network with edge
        :type network: :py:class:`~ndex2.nice_cx_network.NiceCXNetwork`
        :param edgeid:
        :type edgeid: int
        :return: None
        """
        # remove edge
        network.remove_edge(edgeid)

        # remove edge attributes for deleted edge
        net_attrs = network.get_edge_attributes(edgeid)
        if net_attrs is None:
            return
        remove_list = []
        for net_attr in net_attrs:
            remove_list.append(net_attr['n'])
        for attr_name in remove_list:
            logger.debug('Removing ' + str(net_attr['n']) +
                         ' from ' + str(edgeid))
            network.remove_edge_attribute(edgeid, attr_name)
        del remove_list

    def _add_to_edge_map(self, edge_map, edgeid, sourceid, targetid):
        """
        Updates `edge_map` in place with new entry.
        Structure of edge_map
        will be created as follows:

        edge_map[sourceid][targetid] = set(edgeid)

        :param edge_map: edge map to update, should be a empty dict to start
        :type edge_map: dict
        :param edgeid: id of edge
        :type edgeid: int
        :param sourceid: id of source node
        :type sourceid: int
        :param targetid: id of target node
        :type targetid: int
        :return: None
        """
        if not sourceid in edge_map:
            edge_map[sourceid] = {}

        if edge_map[sourceid].get(targetid) is None:
            edge_map[sourceid][targetid] = set()

        if edgeid not in edge_map[sourceid][targetid]:
            edge_map[sourceid][targetid].add(edgeid)

    def _build_edge_map(self, network):
        """
        Iterates through all edges and examines interaction 'i'
        field creating a master dictionary with key set to
        interaction and value set to a dict with following
        structure:


        edge_map[source node id][target node id] = set(edge id)

        :param network: network to extract edges from
        :type network: :py:class:`~ndex2.nice_cx_network.NiceCXNetwork`
        :return: dict with key of interaction and value is a dict of edge_map
        :rtype: dict
        """
        edge_dict = {}
        for k, v in network.get_edges():
            s = v['s']
            t = v['t']
            i = v['i']
            if i not in edge_dict:
                edge_dict[i] = {}
            edge_map = edge_dict[i]

            self._add_to_edge_map(edge_map, k, s, t)

        return edge_dict

    def _convert_attributes_to_dict(self, attr_list):
        """
        This method takes a list of dicts in CX format
        that are for attributes and creates a dict of
        form:

        {'attributename': (value, type)}

        :param attr_list:
        :return:
        """
        attr_dict = {}
        for entry in attr_list:
            attr_dict[entry['n']] = (entry['v'], entry['d'])
        return attr_dict

    def _convert_attributes_to_dict_with_set(self, edge_dict):
        """
        This method takes a dict of format:

        {'attributename': = (value, type)}

        and creates:

        {'attributename': (set(value), type)}

        :param attr_list:
        :return:
        """
        attr_dict = {}
        for key in edge_dict.keys():
            thetype = edge_dict[key][1]
            if thetype == 'list_of_string':
                thevalue = set()
                for curval in edge_dict[key][0]:
                    thevalue.add(curval)
            else:
                thevalue = set()
                thevalue.add(edge_dict[key][0])
            attr_dict[key] = (thevalue, thetype)
        return attr_dict

    def _get_citation_html_frag(self, pubmedurl, pubmedid):
        """

        :param pubmedid:
        :return:
        """
        return '<a target="_blank" href="' +\
               pubmedurl + pubmedid + '">pubmed:' + pubmedid +\
               '</a>'

    def _get_citation_from_edge_dict(self, e_dict):
        """

        :param e_dict:
        :return:
        """
        if e_dict['citation'][1] == 'string':
            if self._pubmedurl is None:
                return ' '
            cite_str = e_dict['citation'][0]
            pubmedid = cite_str[cite_str.index(':') + 1:]
            return self._get_citation_html_frag(self._pubmedurl,
                                                pubmedid) + ' '

        new_cite = ''
        for cite_str in e_dict['citation'][0]:
            pubmedid = cite_str[cite_str.index(':')+1:]
            if self._pubmedurl is None:
                new_cite = new_cite + ' '
            else:
                new_cite = (new_cite +
                            self._get_citation_html_frag(self._pubmedurl,
                                                         pubmedid) + ' ')
        return new_cite

    def _append_attributes_to_dict(self, edge_dict, e_attribs):
        """

        :param edge_dict:
        :param e_attribs:
        :return:
        """
        e_dict = self._convert_attributes_to_dict(e_attribs)
        logger.info('Attributes to e_dict: ' + str(e_dict))
        for key in e_dict.keys():
            if key not in edge_dict:
                return 'Found unexpected new attribute in edge: ' + str(edge_dict)

            thevalue = e_dict[key][0]
            if key == 'sentence':
                cite_str = self._get_citation_from_edge_dict(e_dict) + ' '
                if isinstance(thevalue, list):
                    for valitem in thevalue:
                        edge_dict[key][0].add(cite_str + valitem)
                else:
                    edge_dict[key][0].add(cite_str + thevalue)
            else:
                if isinstance(thevalue, list):
                    for valitem in thevalue:
                        edge_dict[key][0].add(valitem)
                else:
                    edge_dict[key][0].add(thevalue)

    def _update_edge_with_dict(self, network, collapsed_edge, edge_dict):
        """
        Replaces attributes on edge specified by edge id in 'collapsed_edge'
        with values found in 'edge_dict' which is of this structure:

        {'attributename': (set(value), type)}

        With exception of 'direct' which
        is stored as 'boolean', attribute items are added to lists
        and edge attribute type is set to 'list_of_string'

        :param network: Network to update edges on
        :type network: :py:class:`~ndex2.nice_cx_network.NiceCXNetwork`
        :param collapsed_edge: id of future collapsed edge
        :type collapsed_edge: int
        :param edge_dict:
        :type edge_dict: dict
        :return: list of strings denoting problems encountered
        :rtype: list
        """
        issues = []
        for key in edge_dict:
            network.remove_edge_attribute(collapsed_edge, key)
            if key == 'direct':
                if len(edge_dict[key][0]) > 1:
                    issues.append(key +
                                  ' attribute has multiple values: ' +
                                  str(edge_dict[key][0]))

                network.set_edge_attribute(collapsed_edge, key,
                                           edge_dict[key][0].pop(),
                                           type='boolean')
                continue
            network.set_edge_attribute(collapsed_edge, key,
                                       list(edge_dict[key][0]),
                                       type='list_of_string')
        return issues

    def _prepend_citation_to_sentences(self, edge_dict):
        """
        Given a dict of format:

        {'attributename': (value, type)}

        This method prepends the 'sentence' attribute value
        with a href link of citation which is derived from
        'citation' attribute within this dict

        :param edge_dict:
        :return:
        """
        sentence = RedundantEdgeCollapser.SENTENCE
        if sentence not in edge_dict:
            return edge_dict

        if RedundantEdgeCollapser.CITATION not in edge_dict:
            return edge_dict

        cite_str = self._get_citation_from_edge_dict(edge_dict)

        thevalue = edge_dict[sentence][0]

        edge_dict[sentence] = (cite_str + thevalue,
                               edge_dict[sentence][1])

        return edge_dict

    def _collapse_edgeset(self, network, edgeset):
        """
        Given a set of edges collapse these down
        to one edge by converting attributes to a
        list and merging those attributes in.

        :param edgeset:
        :return:
        """
        issues = []
        collapsed_edge = edgeset.pop()
        c_eattrib = network.get_edge_attributes(collapsed_edge)
        c_edict = self._convert_attributes_to_dict(c_eattrib)
        c_edict = self._prepend_citation_to_sentences(c_edict)
        edge_dict = self._convert_attributes_to_dict_with_set(c_edict)
        del c_edict

        logger.info('edge dict: ' + str(edge_dict))
        for edge in edgeset:
            # migrate all attribute data to collapsed_edge
            e_attribs = network.get_edge_attributes(edge)
            msg = self._append_attributes_to_dict(edge_dict, e_attribs)
            if msg is not None and len(msg) > 0:
                issues.append(msg)
            # remove this edge
            self._remove_edge(network, edge)

        msgs = self._update_edge_with_dict(network, collapsed_edge, edge_dict)
        if msgs is not None and len(msgs) > 0:
            issues.extend(msgs)
        del edge_dict
        return issues

    def _iterate_through_edge_map(self, network,
                                  edge_map):
        """
        Iterate through 'edge_dict' which is a dict of this
        structure:

        [source node id][target node id] => set(edge id)

        and collapse edges so there is only one edge
        between two nodes

        :param network:
        :param edge_map:
        :return:
        """
        issues = []
        for srckey in edge_map.keys():
            for tarkey in edge_map[srckey].keys():
                if len(edge_map[srckey][tarkey]) > 0:
                    cur_issues = self._collapse_edgeset(network,
                                                        edge_map[srckey][tarkey])
                    if cur_issues is not None:
                        issues.extend(cur_issues)
        return issues

    def _set_pubmedurl_from_network(self, network):
        """
        Gets the http for pubmed citations from the pubmed entry in
        the @context network attribute setting 'self._pubmedurl' to the
        value extracted

        :param network:
        :type network: :py:class:`~ndex2.nice_cx_network.NiceCXNetwork`
        :return: None
        """
        self._pubmedurl = json.loads(network.get_network_attribute('@context')['v'])['pubmed']

    def update(self, network):
        """
        Examines all edges in network and removes redundant
        edges following this algorithm:

        Remove neighbor-of edges if there is another more descriptive
        edge (anything other then neighbor-of) to the same nodes
        UNLESS this edge has unique citations

        Remove controls-state-change-of edges if there is another more
        descriptive edge (anything other then neighbor-of) to the same
        nodes UNLESS this edge has unique citations

        :param network: network to update
        :type network: :py:class:`~ndex2.nice_cx_network.NiceCXNetwork`
        :return: list of issues as strings encountered
        :rtype: list
        """
        if network is None:
            return ['Network passed in is None']

        self._set_pubmedurl_from_network(network)

        issues = []
        edge_dict = self._build_edge_map(network)
        for key in edge_dict.keys():
            logger.debug('Iterating through edges '
                         'with interaction type: ' + key)
            issues = self._iterate_through_edge_map(network, edge_dict[key])
        del edge_dict
        return issues


class LoadSignorIntoNDEx(object):
    """
    Class to load content
    """

    DISEASE_PATHWAYS = ['ALZHEIMER DISEASE', 'FSGS', 'NOONAN SYNDROME',
                        'PARKINSON DISEASE']

    CANCER_PATHWAYS = ['ACUTE MYELOID LEUKEMIA', 'COLORECTAL CARCINOMA',
                       'GLIOBLASTOMA MULTIFORME', 'LUMINAL BREAST CANCER',
                       'MALIGNANT MELANOMA', 'PROSTATE CANCER',
                       'RHABDOMYOSARCOMA', 'THYROID CANCER']

    def __init__(self, args,
                 downloader,
                 updators=None):
        """

        :param args:
        """
        self._conf_file = args.conf
        self._args = args
        self._profile = args.profile
        self._outdir = os.path.abspath(args.datadir)
        self._user = None
        self._pass = None
        self._server = None
        self._ndex = None
        self._loadplan = None
        self._full_loadplan = None
        self._template = None
        self._net_summaries = None
        self._net_labels = None
        self._downloader = downloader
        self._updators = updators
        self._visibility = args.visibility
        self._indexlevel = args.indexlevel
        self._showcase = not args.disableshowcase

    def _parse_config(self):
            """
            Parses config
            :return:
            """
            ncon = NDExUtilConfig(conf_file=self._conf_file)
            con = ncon.get_config()
            self._user = con.get(self._profile, NDExUtilConfig.USER)
            self._pass = con.get(self._profile, NDExUtilConfig.PASSWORD)
            self._server = con.get(self._profile, NDExUtilConfig.SERVER)

    def _get_user_agent(self):
        """
        Builds user agent string
        :return: user agent string in form of ncipid/<version of this tool>
        :rtype: string
        """
        return 'ndexsignorloader/' + self._args.version

    def _create_ndex_connection(self):
        """
        creates connection to ndex
        :return:
        """
        if self._ndex is None:
            self._ndex = Ndex2(host=self._server, username=self._user,
                               password=self._pass,
                               user_agent=self._get_user_agent())

    def _parse_load_plan(self):
        """

        :return:
        """
        with open(self._args.loadplan, 'r') as f:
            self._loadplan = json.load(f)

        self._full_loadplan = copy.deepcopy(self._loadplan)
        self._full_loadplan['source_plan']['property_columns'].remove({'column_name': 'REGULATOR_LOCATION',
                                                                       'attribute_name': 'location'})
        self._full_loadplan['target_plan']['property_columns'].remove({'column_name': 'TARGET_LOCATION',
                                                                      'attribute_name': 'location'})

    def _load_style_template(self):
        """
        Loads the CX network specified by self._args.style into self._template
        :return:
        """
        if not os.path.isfile(self._args.style):
            res = self._ndex.get_network_as_cx_stream(self._args.style)
            if res.status_code != 200:
                raise NDExLoadSignorError(str(self._args.style) +
                                          ' was not found on the filesystem ' +
                                          'so it was assumed to be a NDEx ' +
                                          ' UUID but got http code: ' +
                                          str(res.status_code) + ' from server')
            self._template = ndex2.create_nice_cx_from_raw_cx(res.json())
        else:
            self._template = ndex2.create_nice_cx_from_file(os.path.abspath(self._args.style))

    def _load_network_labels_for_user(self):
        """
        Gets a dictionary of all networks for a user account in
        this format:

        <value of labels network attribute joined by if list> => <NDEx UUID>

        If a network does not have a **labels** network attribute, the
        network is ignored
        :return:
        """
        net_summaries = self._ndex.get_network_summaries_for_user(self._user)
        self._net_labels = {}
        for nk in net_summaries:
            # nk['externalId']
            val = self._get_labels_network_attribute(netid=nk['externalId'])
            if val is None:
                continue
            if isinstance(val, list):
                valstr = ','.join(val)
            else:
                valstr = str(val)
            self._net_labels[valstr.upper()] = nk['externalId']

    def _get_labels_network_attribute(self, netid=None):
        """
        Queries NDEx for networkAttributes aspect and returns
        value of **labels** network attribute

        :param netid: NDEx network UUID
        :type netid: str
        :return: Value of labels network attribute
        """
        resp = self._ndex.get_network_aspect_as_cx_stream(netid, 'networkAttributes')
        if resp.status_code != 200:
            logger.error('Error getting network attributes for network: ' + str(netid))
            return None
        for n_attr in resp.json():
            if n_attr['n'] != 'labels':
                continue
            return n_attr['v']
        return None

    def _load_network_summaries_for_user(self):
        """
        Gets a dictionary of all networks for user account
        <network name upper cased> => <NDEx UUID>
        :return: dict
        """
        net_summaries = self._ndex.get_network_summaries_for_user(self._user)
        self._net_summaries = {}
        for nk in net_summaries:
            if nk.get('name') is not None:
                self._net_summaries[nk.get('name').upper()] = nk.get('externalId')

    def _wait_for_network_to_be_ready(self, client, netid,
                                      num_retries=10, retry_wait=1):
        """
        Waits for network to be ready for additional modification

        :param client: NDEx client
        :type client: :py:class:`~ndex2.client.Ndex2`
        :param netid: Network UUID
        :type netid: str
        :param num_retries:
        :type num_retries: int
        :param retry_wait:
        :type retry_wait: int or float
        :return: None if network is not ready otherwise the network summary
        :rtype: dict
        """
        retrycount = 1
        while retrycount < num_retries:
            netsum = client.get_network_summary(network_id=netid)
            if netsum['completed'] is True:
                return netsum
            retrycount += 1
            time.sleep(retry_wait)
        return None

    def _get_signor_pathway_relations_df(self, pathway_id,
                                         is_full_pathway=False):
        """
        Loads SIF file `pathway_id`.txt from output directory specified
        in constructor via Pandas.

        :param pathway_id: Prefix of file to load
        :param is_full_pathway: If true it is assumed the pathway_id file
                                is a full network and a custom set of
                                columns is passed to Pandas and a filter
                                is applied removing any entries that are
                                not human
        :return: Pandas data frame of data
        """
        pathway_file_path = os.path.join(self._outdir, pathway_id + '.txt')
        if not os.path.isfile(pathway_file_path):
            raise NDExLoadSignorError(pathway_file_path +
                                      ' file missing.')
        if os.path.getsize(pathway_file_path) < 10:
            raise NDExLoadSignorError(pathway_file_path +
                                      ' looks to be empty')
        usecols = None
        index_col = None
        if is_full_pathway is True:
            logger.info('Full pathway detected, setting columns for pandas')
            usecols = ['entitya', 'typea', 'ida', 'databasea', 'entityb',
                       'typeb', 'idb', 'databaseb', 'effect', 'mechanism',
                       'residue', 'sequence', 'tax_id', 'cell_data',
                       'tissue_data', 'modulator_complex', 'target_complex',
                       'modificationa', 'modaseq', 'modificationb', 'modbseq',
                       'pmid', 'direct', 'notes', 'annotator', 'sentence',
                       'signor_id']
            index_col = False

        with open(pathway_file_path, 'r', encoding='utf-8') as pfp:
            df = pd.read_csv(pfp, dtype=str, na_filter=False,
                             delimiter='\t', names=usecols,
                             index_col=index_col, engine='python')

            # remove rows that are not human
            # (taken from load-content/signor/process_signor.py)
            # not sure if this does that
            if is_full_pathway is True:
                filtered = df[(df["entitya"] != "") &
                              (df["entityb"] != "") &
                              (df["ida"] != "") &
                              (df["idb"] != "")]
                logger.info('Original data frame had: ' +
                            str(len(df.index)) + ' rows and filtered has: ' +
                            str(len(filtered.index)) + ' rows')
                return filtered
            return df

    def _get_signor_pathway_description_df(self, pathway_id):
        pathway_file_path = os.path.join(self._outdir,
                                         pathway_id + '_desc.txt')
        if not os.path.isfile(pathway_file_path):
            raise NDExLoadSignorError(pathway_file_path + ' file missing.')

        with open(pathway_file_path, 'r', encoding='utf-8') as pfp:
            signor_pathway_relations_df = pd.read_csv(pfp, dtype=str,
                                                      na_filter=False,
                                                      delimiter='\t',
                                                      engine='python')

            return signor_pathway_relations_df

    def _add_node_types_in_network_to_report(self, network, report):
        """
        Adds node types to report
        :param network:
        :param report:
        :return: None
        """
        for i, node in network.get_nodes():
            val = network.get_node_attribute_value(i, 'type')
            report.add_nodetype(val)

    def _process_pathway(self, pathway_id, pathway_name):
        """

        :param pathway_id:
        :return:
        """
        is_full_pathway = False
        loadplan = self._loadplan
        if pathway_name.startswith('Signor Complete'):
            is_full_pathway = True
            loadplan = self._full_loadplan

        df = self._get_signor_pathway_relations_df(pathway_id,
                                                   is_full_pathway=is_full_pathway)
        # upcase column names
        rename = {}
        for column_name in df.columns:
            rename[column_name] = column_name.upper()
        df = df.rename(columns=rename)

        network = t2n.convert_pandas_to_nice_cx_with_load_plan(df,
                                                               loadplan)
        report = NetworkIssueReport(pathway_name)

        issues = self._add_pathway_info(network, pathway_id,
                                        is_full_pathway=is_full_pathway,
                                        pathway_name=pathway_name)
        if issues is not None:
            report.addissues('Adding network attributes', issues)
            return report

        if self._updators is not None:
            for updator in self._updators:
                issues = updator.update(network)
                report.addissues(updator.get_description(), issues)

        # apply style to network
        network.apply_style_from_network(self._template)

        network_update_key = self._get_matching_network_on_server_to_update(network)

        self._add_node_types_in_network_to_report(network, report)

        if network_update_key is not None:

            network.update_to(network_update_key, self._server,
                              self._user, self._pass,
                              user_agent=self._get_user_agent())
            netid = network_update_key
        else:
            res = self._ndex.save_new_network(network.to_cx(),
                                              visibility=self._visibility)
            netid = re.sub('^.*/', '', res)

        waitres = self._wait_for_network_to_be_ready(self._ndex, netid)
        if waitres is not None:
            self._ndex.set_network_system_properties(netid, {'showcase': self._showcase,
                                                             'visibility': self._visibility,
                                                             'index_level': self._indexlevel})
        else:
            report.addissues('Updating index, showcase, visibility',
                             ['Network ' + str(network.get_name()) + ' not ready'])

        return report

    def _get_matching_network_on_server_to_update(self, network):
        """
        Searches for matching network first by **labels** network
        attribute and then by name of network if **labels** does
        not match

        :param network:
        :type network: :py:class:`~ndex2.nice_cx_network.NiceCXNetwork`
        :return: NDEx UUID of network to update or None
        :rtype: str
        """
        n_attr = network.get_network_attribute('labels')
        if n_attr is None:
            # attempt to match by name
            return self._net_summaries.get(network.get_name().upper())

        val = n_attr['v']
        if isinstance(val, list):
            valstr = ','.join(val)
        else:
            valstr = str(val)

        return self._net_labels.get(valstr.upper())

    def _set_generatedby_in_network_attributes(self, network):
        """
        Sets the network attribute :py:const:`GENERATED_BY_ATTRIB`
        with ndexncipidloader <VERSION>
        :param network: network to add attribute
        :type :py:class:`~ndex2.nice_cx_network.NiceCXNetwork`
        :return: None
        """
        network.set_network_attribute(GENERATED_BY_ATTRIB,
                                      '<a href="https://github.com/'
                                      'ndexcontent/ndexsignorloader"'
                                      '>ndexsignorloader ' +
                                      str(ndexsignorloader.__version__) +
                                      '</a>')

    def _set_normalization_version(self, network):
        """
        Sets the network attribute :py:const:`NORMALIZATIONVERSION`
        with 0.1
        :param network: network to add attribute
        :type :py:class:`~ndex2.nice_cx_network.NiceCXNetwork`
        :return: None
        """
        network.set_network_attribute(NORMALIZATIONVERSION_ATTRIB, '0.1')

    def _set_wasderivedfrom(self, network, pathway_id,
                            is_full_pathway=False):
        """
        Sets the 'prov:wasDerivedBy' network attribute to the
        ftp location containing the OWL file for this network.
        The ftp information is pulled from :py:const:`DEFAULT_FTP_HOST` and
         :py:const:`DEFAULT_FTP_DIR` and the owl file name is the
         name of the network with .owl.gz appended

        :param network: network to add attribute
        :type :py:class:`~ndex2.nice_cx_network.NiceCXNetwork`
        :return: None
        """
        derivedurl = 'https://signor.uniroma2.it'
        if is_full_pathway is False:
            derivedurl = derivedurl +\
                         '/pathway_browser.php?organism=&' +\
                         'pathway_list=' + str(pathway_id)

        network.set_network_attribute(DERIVED_FROM_ATTRIB, derivedurl)

    def _set_type(self, network,
                  is_human_fullpathway=False):
        """
        Sets type network attribute adding a list with 'pathway' and an additional
        value based on name of network.
        :param network: network to update
        :type network: :py:class:`~ndex2.nice_cx_network.NiceCXNetwork`
        :return: None
        """
        typedata = []
        if is_human_fullpathway is True:
            typedata.append('interactome')

        typedata.append('pathway')

        if network.get_name().upper() in LoadSignorIntoNDEx.DISEASE_PATHWAYS:
            typedata.append("Disease Pathway")
        elif network.get_name().upper() in LoadSignorIntoNDEx.CANCER_PATHWAYS:
            typedata.append("Cancer Pathway")
        else:
            typedata.append("Signalling Pathway")
        network.set_network_attribute('networkType', typedata, type='list_of_string')

    def _set_iconurl(self, network):
        """
        Sets the network attribute :py:const:`ICONURL_ATTRIB` with
        value from self._args.iconurl passed in constructor

        :param network: network to add attribute
        :type :py:class:`~ndex2.nice_cx_network.NiceCXNetwork`
        :return:
        """
        network.set_network_attribute(ICONURL_ATTRIB,
                                      self._args.iconurl)

    def _set_edgecollapse_notes(self, network):
        """
        Adds network attribute named :py:const:`NOTES_ATTRIB` with
        value set to description of edge collapse

        :param network: network to add attribute
        :type :py:class:`~ndex2.nice_cx_network.NiceCXNetwork`
        :return:
        """
        if self._args.edgecollapse:
            network.set_network_attribute(NOTES_ATTRIB,
                                          'Edges have been collapsed with '
                                          'attributes converted to lists with'
                                          ' exception of direct attribute')

    def _add_pathway_info(self, network, pathway_id,
                          is_full_pathway=False,
                          pathway_name=None):
        """
        Adds network
        :param network:
        :param network_id:
        :param cytoscape_visual_properties_template_id: UUID of NDEx
               network to
               extract various network attributes such as description,
               rightsHolder, reference, rights

        :return:
        """
        is_human_fullpathway = False

        if is_full_pathway is False:
            try:
                dataframe = self._get_signor_pathway_description_df(pathway_id)
            except Exception as e:
                logger.warning('Skipping ' + pathway_id +
                               ' due to error parsing for pathway: ' +
                               pathway_id + ' : ' +
                               str(e), e)
                return ['Error parsing description for pathway: ' +
                        pathway_id + ' : ' + str(e)]
            if dataframe is None:
                logger.warning('Skipping ' + pathway_id)
                return ['No description skipping ' + pathway_id]
            if not pd.isnull(dataframe.iat[0, 1]):
                network.set_name(dataframe.iat[0, 1])
            if not pd.isnull(dataframe.iat[0, 0]):
                network.set_network_attribute("labels", [dataframe.iat[0, 0]],
                                              type='list_of_string')
            if not pd.isnull(dataframe.iat[0, 3]):
                auth_val = dataframe.iat[0, 3]
                if auth_val is not None and len(auth_val) > 0:
                    network.set_network_attribute("author",
                                                  dataframe.iat[0, 3])
            if not pd.isnull(dataframe.iat[0, 2]):
                network.set_network_attribute("description",
                                              '%s' % (dataframe.iat[0, 2]))
            network.set_network_attribute("organism", "Homo Sapiens (human)")
        else:
            logger.info("Full pathway detected: " + str(pathway_name))
            network.set_name(pathway_name)

            net_organism = 'Unknown'
            if 'Human' in pathway_id:
                net_organism = 'Human'
                network.set_network_attribute("organism", "Homo sapiens (human)")
                is_human_fullpathway = True
            elif 'Rat' in pathway_id:
                net_organism = 'Rat'
                network.set_network_attribute("organism", "Rattus norvegicus (rat)")
            elif 'Mouse' in pathway_id:
                net_organism = 'Mouse'
                network.set_network_attribute("organism", "Mus musculus (mouse)")
            else:
                logger.error('No matching organism found for: ' + pathway_id)

            network.set_network_attribute('description',
                                          'This network contains all the ' +
                                          net_organism +
                                          ' interactions currently available '
                                          'in SIGNOR')

        network.set_network_attribute('rightsHolder', 'Prof. Gianni Cesareni')
        network.set_network_attribute('rights',
                                      'Attribution-ShareAlike 4.0 '
                                      'International (CC BY-SA 4.0)')

        network.set_network_attribute("reference",
                                      '<div>Perfetto L., <i>et al.</i></div>'
                                      '<div><b>SIGNOR: a database of causal '
                                      'relationships between biological '
                                      'entities</b><i>.</i></div><div>Nucleic '
                                      'Acids Res. 2016 Jan 4;44(D1):D548-54'
                                      '</div><div><span><a href="'
                                      'https://doi.org/10.1093/nar/gkv1048'
                                      '" target="_blank">'
                                      'doi: 10.1093/nar/gkv1048</a></span>'
                                      '</div>')

        dtime = datetime.now().strftime('%d-%b-%Y')
        network.set_network_attribute("version", dtime)

        # set type network attribute
        self._set_type(network,
                       is_human_fullpathway=is_human_fullpathway)

        if is_full_pathway is True:
            # set iconurl
            self._set_iconurl(network)

        # set provenance for network
        self._set_generatedby_in_network_attributes(network)

        # set normalization version for network
        self._set_normalization_version(network)

        # set was derived from
        self._set_wasderivedfrom(network, str(pathway_id),
                                 is_full_pathway=is_full_pathway)

        # writes out network attribute describing
        # edge collapse if it was performed
        self._set_edgecollapse_notes(network)
        return None

    def run(self):
        """
        Runs content loading for NDEx Signor Content Loader
        :param theargs:
        :return:
        """
        self._parse_config()
        logger.debug('Parsed config: ' + self._user)
        self._parse_load_plan()
        self._create_ndex_connection()
        self._load_network_summaries_for_user()
        self._load_network_labels_for_user()
        self._load_style_template()
        report_list = []

        pathway_map = self._downloader.get_pathways_map()
        if pathway_map is None:
            logger.error('Pathway map came back as None')
            return 1
        for key in pathway_map.keys():
            logger.info('Processing ' + key + ' => ' + pathway_map[key])
            try:
                report_list.append(self._process_pathway(key, pathway_map[key]))
            except NDExLoadSignorError as ne:
                logger.exception('Unable to load pathway: ' + key +
                                 ' => ' + pathway_map[key])

        # process full pathways
        if self._args.skipfull is False:
            for orgname in ['Human', 'Mouse', 'Rat']:
                pname = 'Signor Complete - ' + orgname
                logger.info('Processing full ' + pname)
                try:
                    report_list.append(self._process_pathway('full_' + orgname,
                                                             pname))
                except NDExLoadSignorError as ne:
                    logger.exception('Unable to load pathway: ' +
                                     'full_' + orgname + '.txt')

        node_type = set()
        for entry in report_list:
            for nt in entry.get_nodetypes():
                node_type.add(nt)
            sys.stdout.write(entry.get_fullreport_as_string())

        sys.stdout.write('Node Types Found in all networks:\n')
        for entry in node_type:
            sys.stdout.write('\t' + entry + '\n')

        return 0


def main(args):
    """
    Main entry point for program
    :param args:
    :return:
    """
    desc = """
    Version {version}

    Loads NDEx Signor Content Loader data into NDEx (http://ndexbio.org).
    
    To connect to NDEx server a configuration file must be passed
    into --conf parameter. If --conf is unset the configuration 
    path ~/{confname} is examined. 
         
    The configuration file should be formatted as follows:
         
    [<value in --profile (default ncipid)>]
         
    {user} = <NDEx username>
    {password} = <NDEx password>
    {server} = <NDEx server(omit http) ie public.ndexbio.org>
    
    For more information on what operations are performed
    visit: https://github.com/ndexcontent/ndexsignorloader

    """.format(confname=NDExUtilConfig.CONFIG_FILE,
               user=NDExUtilConfig.USER,
               password=NDExUtilConfig.PASSWORD,
               server=NDExUtilConfig.SERVER,
               version=ndexsignorloader.__version__)
    theargs = _parse_arguments(desc, args[1:])
    theargs.program = args[0]
    theargs.version = ndexsignorloader.__version__

    try:
        _setup_logging(theargs)
        datadir = os.path.abspath(theargs.datadir)
        downloader = SignorDownloader(theargs.signorurl,
                                      datadir)
        if theargs.skipdownload is False:
            downloader.download_data()

        updators = [DirectEdgeAttributeUpdator(),
                    UpdatePrefixesForNodeRepresents(),
                    NodeLocationUpdator(),
                    NodeMemberUpdator(downloader.get_proteinfamily_map(),
                                      downloader.get_complexes_map()),
                    InvalidEdgeCitationRemover()]

        # add edge collapse updator if flag is set
        if theargs.edgecollapse is True:
            updators.append(RedundantEdgeCollapser())

        updators.append(SpringLayoutUpdator())

        loader = LoadSignorIntoNDEx(theargs, downloader,
                                    updators=updators)
        return loader.run()
    except Exception as e:
        logger.exception('Caught exception')
        return 2
    finally:
        logging.shutdown()


if __name__ == '__main__':  # pragma: no cover
    sys.exit(main(sys.argv))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `NodeLocationUpdator` class."""

import os
import tempfile
import shutil

import unittest
from ndex2.nice_cx_network import NiceCXNetwork
from ndexsignorloader.ndexloadsignor import NodeLocationUpdator


class TestNodeLocationUpdator(unittest.TestCase):
    """Tests for `NodeLocationUpdator` class."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_get_description(self):
        updator = NodeLocationUpdator()
        self.assertEqual('Replace any empty node location attribute '
                         'values with cytoplasm',
                         updator.get_description())

    def test_update_network_is_none(self):
        updator = NodeLocationUpdator()
        self.assertEqual(['network is None'],
                         updator.update(None))

    def test_update_network_empty(self):
        updator = NodeLocationUpdator()
        net = NiceCXNetwork()
        self.assertEqual([],
                         updator.update(net))

    def test_update_network_containing_all_types(self):
        updator = NodeLocationUpdator()
        net = NiceCXNetwork()

        comp_attr = NodeLocationUpdator.LOCATION
        no_attr = net.create_node('somenode', node_represents='rep1')

        e_attr = net.create_node('somenode2',
                                 node_represents='uniprot:rep2')
        net.set_node_attribute(e_attr,
                               NodeLocationUpdator.LOCATION,
                               '')

        o_attr = net.create_node('somenode2',
                                 node_represents='uniprot:rep2')
        net.set_node_attribute(o_attr,
                               NodeLocationUpdator.LOCATION,
                               'blah')

        ph_attr = net.create_node('anothernode', node_represents='rep3')

        net.set_node_attribute(ph_attr,
                               NodeLocationUpdator.LOCATION,
                               NodeLocationUpdator.PHENOTYPESLIST)

        self.assertEqual([],
                         updator.update(net))

        self.assertEqual(NodeLocationUpdator.CYTOPLASM,
                         net.get_node_attribute(no_attr, comp_attr)['v'])

        self.assertEqual(NodeLocationUpdator.CYTOPLASM,
                         net.get_node_attribute(e_attr, comp_attr)['v'])

        self.assertEqual('blah',
                         net.get_node_attribute(o_attr, comp_attr)['v'])

        self.assertEqual('',
                         net.get_node_attribute(ph_attr, comp_attr)['v'])




#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `RedundantEdgeCollapser` class."""

import os
import tempfile
import shutil
import json
import unittest
from ndex2.nice_cx_network import NiceCXNetwork
from ndexsignorloader.ndexloadsignor import RedundantEdgeCollapser


class TestRedundantEdgeCollapser(unittest.TestCase):
    """Tests for `RedundantEdgeCollapser` class."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_get_description(self):
        collapser = RedundantEdgeCollapser()
        self.assertEqual('Collapses redundant edges',
                         collapser.get_description())

    def test_get_citation_html_frag(self):
        collapser = RedundantEdgeCollapser()
        res = collapser._get_citation_html_frag('pubmedurl/', 'pubmedid')
        self.assertEqual('<a target="_blank" href="pubmedurl/pubmedid">'
                         'pubmed:pubmedid</a>',
                         res)

    def test_remove_edge(self):
        collapser = RedundantEdgeCollapser()
        net = NiceCXNetwork()

        # test remove on non existant edge
        collapser._remove_edge(net, 10)

        # test removing edge with no attributes
        eid = net.create_edge(edge_source=0, edge_target=1,
                              edge_interaction='needs')
        self.assertEqual('needs',
                         net.get_edge(eid)['i'])
        collapser._remove_edge(net, eid)
        self.assertEqual(None,
                         net.get_edge(eid))

        # test removing edge with attributes
        eid = net.create_edge(edge_source=0, edge_target=1,
                              edge_interaction='needs')

        net.set_edge_attribute(eid, 'foo', 'someval')
        net.set_edge_attribute(eid, 'foo2', 'someval2')

        self.assertEqual('needs',
                         net.get_edge(eid)['i'])
        self.assertEqual('someval',
                         net.get_edge_attribute(eid, 'foo')['v'])
        self.assertEqual('someval2',
                         net.get_edge_attribute(eid, 'foo2')['v'])
        collapser._remove_edge(net, eid)
        self.assertEqual(None,
                         net.get_edge(eid))
        self.assertEqual((None, None),
                         net.get_edge_attribute(eid, 'foo'))
        self.assertEqual((None, None),
                         net.get_edge_attribute(eid, 'foo2'))

    def test_add_edge_to_map(self):
        collapser = RedundantEdgeCollapser()

        # test adding new entry
        edge_map = {}
        collapser._add_to_edge_map(edge_map, 0, 1, 2)
        self.assertEqual({0}, edge_map[1][2])

        # test adding duplicate
        collapser._add_to_edge_map(edge_map, 0, 1, 2)
        self.assertEqual({0}, edge_map[1][2])

        # test adding new entry to existing dict
        collapser._add_to_edge_map(edge_map, 3, 1, 2)
        self.assertEqual({0, 3}, edge_map[1][2])

    def test_build_edge_map(self):
        collapser = RedundantEdgeCollapser()

        # try on empty network
        net = NiceCXNetwork()
        edge_dict = collapser._build_edge_map(net)
        self.assertEqual({}, edge_dict)

        # try on single edge network
        net.create_edge(edge_source=0, edge_target=1,
                        edge_interaction='something')

        edge_dict = collapser._build_edge_map(net)
        self.assertEqual({'something': {0: {1: {0}}}}, edge_dict)

        # add another edge same interaction
        net.create_edge(edge_source=0, edge_target=1,
                        edge_interaction='something')
        edge_dict = collapser._build_edge_map(net)
        self.assertEqual({'something': {0: {1: {0, 1}}}}, edge_dict)

        # add another edge different interaction
        net.create_edge(edge_source=0, edge_target=1,
                        edge_interaction='foo')
        edge_dict = collapser._build_edge_map(net)
        self.assertEqual({0: {1: {0, 1}}}, edge_dict['something'])
        self.assertEqual({0: {1: {2}}}, edge_dict['foo'])

    def test_convert_attributes_to_dict(self):
        collapser = RedundantEdgeCollapser()

        # test on single item attribute list
        attr_list = [{'n': 'name1', 'v': 'value1',
                      'd': 'string'}]
        attr_dict = collapser._convert_attributes_to_dict(attr_list)

        self.assertEqual(('value1', 'string'), attr_dict['name1'])

        # try a second item
        attr_list.append({'n': 'name2', 'v': ['value2'],
                          'd': 'list_of_string'})

        attr_dict = collapser._convert_attributes_to_dict(attr_list)

        self.assertEqual(('value1', 'string'), attr_dict['name1'])
        self.assertEqual((['value2'], 'list_of_string'),
                         attr_dict['name2'])

    def test_convert_attributes_to_dict_with_set(self):
        collapser = RedundantEdgeCollapser()

        # test on single item attribute list
        edge_dict = {'name1': ('value1', 'string')}
        attr_dict = collapser._convert_attributes_to_dict_with_set(edge_dict)

        self.assertEqual(({'value1'}, 'string'), attr_dict['name1'])

        # try a second item
        edge_dict['name2'] = (['value2'], 'list_of_string')

        attr_dict = collapser._convert_attributes_to_dict_with_set(edge_dict)

        self.assertEqual(({'value1'}, 'string'), attr_dict['name1'])
        self.assertEqual(({'value2'}, 'list_of_string'),
                         attr_dict['name2'])

    def test_get_citation_from_edge_dict(self):
        collapser = RedundantEdgeCollapser()

        # single citation pubmedurl is None
        edge_dict = {'citation': (['pubmed:123'], 'list_of_string')}
        res = collapser._get_citation_from_edge_dict(edge_dict)
        self.assertEqual(' ', res)

        # multiple citation pubmedurl is None
        edge_dict = {'citation': (['pubmed:123',
                                   'pubmed:456'], 'list_of_string')}
        res = collapser._get_citation_from_edge_dict(edge_dict)
        self.assertEqual('  ', res)

        # single citation pubmedurl is set
        net = NiceCXNetwork()
        ctext = {'pubmed': 'http://p/'}
        net.set_network_attribute('@context', values=json.dumps(ctext))
        edge_dict = {'citation': (['pubmed:123'], 'list_of_string')}
        collapser._set_pubmedurl_from_network(net)
        res = collapser._get_citation_from_edge_dict(edge_dict)
        self.assertEqual('<a target="_blank" '
                         'href="http://p/123">pubmed:123</a> ', res)

        # multiple citation pubmedurl is set
        edge_dict = {'citation': (['pubmed:123',
                                   'pubmed:456'], 'list_of_string')}
        res = collapser._get_citation_from_edge_dict(edge_dict)
        self.assertEqual('<a target="_blank" '
                         'href="http://p/123">pubmed:123</a> '
                         '<a target="_blank" href="http://p/456">pubmed:'
                         '456</a> ', res)

    def test_prepend_citation_to_sentences(self):
        collapser = RedundantEdgeCollapser()
        net = NiceCXNetwork()
        ctext = {'pubmed': 'http://p/'}
        net.set_network_attribute('@context', values=json.dumps(ctext))
        collapser._set_pubmedurl_from_network(net)

        res = collapser._prepend_citation_to_sentences({})
        self.assertEqual({}, res)
        res = collapser._prepend_citation_to_sentences({'sentence':
                                                        ('hi', 'string')})
        self.assertEqual({'sentence':
                          ('hi', 'string')}, res)
        edge_dict = {'citation': (['pubmed:123'], 'list_of_string'),
                     'sentence': ('sentence2', 'string')}
        res = collapser._prepend_citation_to_sentences(edge_dict)
        self.assertEqual('<a target="_blank" href="http://p/123">'
                         'pubmed:123</a> sentence2',
                         res['sentence'][0])
        self.assertEqual('string', res['sentence'][1])

    def test_update_edge_with_dict(self):
        collapser = RedundantEdgeCollapser()
        net = NiceCXNetwork()

        eid = net.create_edge(edge_source=0, edge_target=1,
                              edge_interaction='something')
        net.set_edge_attribute(eid, 'sentence', 'hi', type='string')

        edge_dict = {'citation': (set(['pubmed:123']), 'list_of_string'),
                     'sentence': (set(['sentence1', 'sentence2']), 'string'),
                     'direct': (set([True, False]), 'boolean')}

        res = collapser._update_edge_with_dict(net, eid, edge_dict)

        self.assertTrue('direct attribute has multiple values:' in res[0])

        edata = net.get_edge_attribute(eid, 'citation')
        self.assertEqual('list_of_string', edata['d'])
        self.assertEqual(['pubmed:123'], edata['v'])

        edata = net.get_edge_attribute(eid, 'sentence')
        self.assertEqual('list_of_string', edata['d'])
        self.assertTrue('sentence1' in edata['v'])
        self.assertTrue('sentence2' in edata['v'])
        self.assertEqual(2, len(edata['v']))

        edata = net.get_edge_attribute(eid, 'direct')
        self.assertEqual('boolean', edata['d'])
        self.assertEqual(False, edata['v'])

    def test_collapse_edgeset(self):
        collapser = RedundantEdgeCollapser()

        net = NiceCXNetwork()
        ctext = {'pubmed': 'http://p/'}
        net.set_network_attribute('@context', values=json.dumps(ctext))
        collapser._set_pubmedurl_from_network(net)

        eid = net.create_edge(edge_source=0, edge_target=1,
                              edge_interaction='something')

        net.set_edge_attribute(eid, 'sentence', 'sent1', type='string')
        net.set_edge_attribute(eid, 'direct', True, type='boolean')
        net.set_edge_attribute(eid, 'citation', 'pubmed:123', type='string')

        eidtwo = net.create_edge(edge_source=0, edge_target=1,
                              edge_interaction='something')
        net.set_edge_attribute(eidtwo, 'sentence', 'sent2', type='string')
        net.set_edge_attribute(eidtwo, 'direct', True, type='boolean')
        net.set_edge_attribute(eidtwo, 'citation', 'pubmed:456', type='string')

        issues = collapser._collapse_edgeset(net, set([eid, eidtwo]))
        self.assertEqual([], issues)

        edata = net.get_edge_attribute(eid, 'sentence')
        self.assertEqual(2, len(edata['v']))

        self.assertTrue('<a target="_blank" href="http://p/456">'
                        'pubmed:456</a>  sent2' in edata['v'])
        self.assertTrue('<a target="_blank" href="http://p/123">'
                        'pubmed:123</a> sent1' in edata['v'])

    def test_update_none_passed_in(self):
        collapser = RedundantEdgeCollapser()
        res = collapser.update(None)
        self.assertEqual(['Network passed in is None'], res)

    def test_update(self):
        collapser = RedundantEdgeCollapser()
        net = NiceCXNetwork()
        ctext = {'pubmed': 'http://p/'}
        net.set_network_attribute('@context', values=json.dumps(ctext))
        collapser._set_pubmedurl_from_network(net)

        eid = net.create_edge(edge_source=0, edge_target=1,
                              edge_interaction='something')

        net.set_edge_attribute(eid, 'sentence', 'sent1', type='string')
        net.set_edge_attribute(eid, 'direct', True, type='boolean')
        net.set_edge_attribute(eid, 'citation', 'pubmed:123', type='string')

        eidtwo = net.create_edge(edge_source=0, edge_target=1,
                                 edge_interaction='something')
        net.set_edge_attribute(eidtwo, 'sentence', 'sent2', type='string')
        net.set_edge_attribute(eidtwo, 'direct', True, type='boolean')
        net.set_edge_attribute(eidtwo, 'citation', 'pubmed:456', type='string')
        res = collapser.update(net)

        self.assertEqual([], res)

        edata = net.get_edge_attribute(eid, 'sentence')
        self.assertEqual(2, len(edata['v']))

        self.assertTrue('<a target="_blank" href="http://p/456">'
                        'pubmed:456</a>  sent2' in edata['v'])
        self.assertTrue('<a target="_blank" href="http://p/123">'
                        'pubmed:123</a> sent1' in edata['v'])






#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `LoadSignorIntoNDex` class."""

import os
import tempfile
import shutil

import unittest
from ndexsignorloader.ndexloadsignor import LoadSignorIntoNDEx
from ndexsignorloader import ndexloadsignor
from ndex2.nice_cx_network import NiceCXNetwork


class FakeArgs(object):
    pass


class TestLoadSignorIntoNDex(unittest.TestCase):
    """Tests for `LoadSignorIntoNDex` class."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_parse_config(self):
        temp_dir = tempfile.mkdtemp()
        try:
            cfile = os.path.join(temp_dir, 'conf')
            with open(cfile, 'w') as f:
                f.write('[hi]\n')
                f.write('user=theuser\n')
                f.write('password=thep\n')
                f.write('server=thes\n')
            fargs = FakeArgs()
            fargs.conf = cfile
            fargs.profile = 'hi'
            fargs.datadir = temp_dir
            fargs.visibility = 'PUBLIC'
            fargs.indexlevel = 'ALL'
            fargs.disableshowcase = False

            loader = LoadSignorIntoNDEx(fargs, None)
            loader._parse_config()
            self.assertEqual('theuser', loader._user)
            self.assertEqual('thep', loader._pass)
            self.assertEqual('thes', loader._server)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_user_agent(self):
        fargs = FakeArgs()
        fargs.conf = '/ho'
        fargs.profile = 'hi'
        fargs.datadir = '/'
        fargs.visibility = 'PUBLIC'
        fargs.version = '0.3.0'
        fargs.indexlevel = 'ALL'
        fargs.disableshowcase = False

        loader = LoadSignorIntoNDEx(fargs, None)
        res = loader._get_user_agent()
        self.assertEqual('ndexsignorloader/0.3.0', res)

    def test_set_edgecollapse_notes_with_edgecollapse_true(self):
        fargs = FakeArgs()
        fargs.conf = '/ho'
        fargs.profile = 'hi'
        fargs.datadir = '/'
        fargs.visibility = 'PUBLIC'
        fargs.edgecollapse = True
        fargs.indexlevel = 'ALL'
        fargs.disableshowcase = False

        loader = LoadSignorIntoNDEx(fargs, None)
        net = NiceCXNetwork()
        loader._set_edgecollapse_notes(net)
        res = net.get_network_attribute(ndexloadsignor.NOTES_ATTRIB)
        self.assertTrue('Edges have been' in res['v'])

    def test_set_edgecollapse_notes_with_edgecollapse_false(self):
        fargs = FakeArgs()
        fargs.conf = '/ho'
        fargs.profile = 'hi'
        fargs.datadir = '/'
        fargs.visibility = 'PUBLIC'
        fargs.edgecollapse = False
        fargs.indexlevel = 'ALL'
        fargs.disableshowcase = False

        loader = LoadSignorIntoNDEx(fargs, None)
        net = NiceCXNetwork()
        loader._set_edgecollapse_notes(net)
        res = net.get_network_attribute(ndexloadsignor.NOTES_ATTRIB)
        self.assertEqual(None, res)

    def test_wasderivedfrom(self):
        fargs = FakeArgs()
        fargs.conf = 'hi'
        fargs.profile = 'profile'
        fargs.datadir = '/foo'
        fargs.visibility = 'PUBLIC'
        fargs.indexlevel = 'ALL'
        fargs.disableshowcase = False

        loader = LoadSignorIntoNDEx(fargs, None)
        net = NiceCXNetwork()
        loader._set_wasderivedfrom(net, 'foo')
        res = net.get_network_attribute(ndexloadsignor.DERIVED_FROM_ATTRIB)
        self.assertEqual('https://signor.uniroma2.it/pathway_browser.php?'
                         'organism=&pathway_list=foo', res['v'])

        # try with is_full_pathway false
        net = NiceCXNetwork()
        loader._set_wasderivedfrom(net, 'foo',
                                   is_full_pathway=False)
        res = net.get_network_attribute(ndexloadsignor.DERIVED_FROM_ATTRIB)
        self.assertEqual('https://signor.uniroma2.it/pathway_browser.php?'
                         'organism=&pathway_list=foo', res['v'])

        # try with is_full_pathway true
        net = NiceCXNetwork()
        loader._set_wasderivedfrom(net, 'foo',
                                   is_full_pathway=True)
        res = net.get_network_attribute(ndexloadsignor.DERIVED_FROM_ATTRIB)
        self.assertEqual('https://signor.uniroma2.it', res['v'])

    def test_set_iconurl(self):
        fargs = FakeArgs()
        fargs.conf = 'hi'
        fargs.profile = 'profile'
        fargs.datadir = '/foo'
        fargs.iconurl = 'hi'
        fargs.visibility = 'PUBLIC'
        fargs.indexlevel = 'ALL'
        fargs.disableshowcase = False

        loader = LoadSignorIntoNDEx(fargs, None)
        net = NiceCXNetwork()
        loader._set_iconurl(net)
        res = net.get_network_attribute('__iconurl')
        self.assertEqual('hi', res['v'])

    def test_set_type(self):
        fargs = FakeArgs()
        fargs.conf = 'hi'
        fargs.profile = 'profile'
        fargs.datadir = '/foo'
        fargs.visibility = 'PUBLIC'
        fargs.indexlevel = 'ALL'
        fargs.disableshowcase = False

        loader = LoadSignorIntoNDEx(fargs, None)

        net = NiceCXNetwork()

        # test default
        net.set_name('foo')
        loader._set_type(net)
        res = net.get_network_attribute('networkType')
        self.assertEqual(['pathway', 'Signalling Pathway'], res['v'])

        # test disease pathway
        net.set_name('fsgs')
        loader._set_type(net)
        res = net.get_network_attribute('networkType')
        self.assertEqual(['pathway', 'Disease Pathway'], res['v'])

        # test cancer pathway
        net.set_name('prostate cancer')
        loader._set_type(net)
        res = net.get_network_attribute('networkType')
        self.assertEqual(['pathway', 'Cancer Pathway'], res['v'])

        #test interactome True
        net.set_name('some name')
        loader._set_type(net, is_human_fullpathway=True)
        res = net.get_network_attribute('networkType')
        self.assertEqual(['interactome', 'pathway',
                          'Signalling Pathway'], res['v'])

    def test_add_pathway_info(self):
        fargs = FakeArgs()
        fargs.conf = 'hi'
        fargs.iconurl = 'http://myicon'
        fargs.edgecollapse = False
        fargs.profile = 'profile'
        fargs.datadir = '/foo'
        fargs.visibility = 'PUBLIC'
        fargs.indexlevel = 'ALL'
        fargs.disableshowcase = False

        loader = LoadSignorIntoNDEx(fargs, None)

        net = NiceCXNetwork()
        loader._add_pathway_info(net, 'Human', True, 'mypathway')

        net_attr = net.get_network_attribute('description')
        self.assertEqual('This network contains all the Human' +
                         ' interactions currently available ' +
                         'in SIGNOR', net_attr['v'])
        net_attr = net.get_network_attribute('organism')
        self.assertEqual('Homo sapiens (human)', net_attr['v'])
        net_attr = net.get_network_attribute('rightsHolder')
        self.assertEqual('Prof. Gianni Cesareni', net_attr['v'])
        net_attr = net.get_network_attribute('reference')
        self.assertTrue('<a href="https://doi.org/10.1093/nar/gkv1048" '
                        'target="_blank">doi: 10.1093'
                        '/nar/gkv1048</a>' in net_attr['v'])




#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""
import os
import re
from setuptools import setup, find_packages


with open(os.path.join('ndexsignorloader', '__init__.py')) as ver_file:
    for line in ver_file:
        if line.startswith('__version__'):
            version=re.sub("'", "", line[line.index("'"):])

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = ['ndex2',
                'ndexutil',
                'requests',
                'pandas']

setup_requirements = [ ]

test_requirements = ['requests-mock',
                     'mock']

setup(
    author="Chris Churas",
    author_email='contact@ndexbio.org',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    description="Loads SIGNOR data into NDEx",
    install_requires=requirements,
    license="BSD license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='ndexsignorloader',
    name='ndexsignorloader',
    packages=find_packages(include=['ndexsignorloader']),
    package_dir={'ndexsignorloader': 'ndexsignorloader'},
    package_data={'ndexsignorloader': ['loadplan.json',
                                       'style.cx']},
    scripts=[ 'ndexsignorloader/ndexloadsignor.py'],
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/ndexcontent/ndexsignorloader',
    version=version,
    zip_safe=False,
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `SignorDownloader` class."""

import os
import tempfile
import shutil
from mock import MagicMock

import unittest
import requests_mock
from ndex2.nice_cx_network import NiceCXNetwork
from ndexsignorloader.ndexloadsignor import SignorDownloader
from ndexsignorloader.ndexloadsignor import NDExLoadSignorError


class TestSignorDownloader(unittest.TestCase):
    """Tests for `NodeMemberUpdator` class."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def get_fake_signor_pf_text(self):
        return """"SIGNOR ID";"PROT. FAMILY NAME";"LIST OF ENTITIES"
SIGNOR-PF1;ERK1/2;"P27361,  P28482"
SIGNOR-PF2;LPAR;"Q9HBW0,  Q9UBY5,  Q92633"
SIGNOR-PF3;Ggamma;"P63211,  P61952,  O14610,  P50151,  P63218,  Q9UBI6,  Q9UK08,  P59768,  P50150,  Q9P2W3,  O60262,  P63215"
SIGNOR-PF4;Gbeta;"P27361,  P28482"
"""

    def write_fake_signor_pf_file(self, destfile):
        with open(destfile, 'w') as f:
            f.write(self.get_fake_signor_pf_text())

    def get_fake_signor_complexes_text(self):
        return """"SIGNOR ID";"COMPLEX NAME";"LIST OF ENTITIES"
SIGNOR-C1;NFY;"P23511,  P25208,  Q13952"
SIGNOR-C2;mTORC2;"P68104,  Q8TB45"
"""

    def write_fake_signor_complexes_file(self, destfile):
        with open(destfile, 'w') as f:
            f.write(self.get_fake_signor_complexes_text())

    def test_get_pathway_list_file(self):
        dloader = SignorDownloader(None, 'f')
        self.assertEqual(os.path.join('f',
                                      SignorDownloader.PATHWAY_LIST_FILE),
                                      dloader.get_pathway_list_file())

    def test_get_complexes_file(self):
        dloader = SignorDownloader(None, 'f')
        self.assertEqual(os.path.join('f',
                                      SignorDownloader.COMPLEXES_FILE),
                                      dloader.get_complexes_file())

    def test_get_proteinfamily_map(self):
        temp_dir = tempfile.mkdtemp()
        try:
            dloader = SignorDownloader(None, temp_dir)
            pfile = dloader.get_proteinfamily_file()
            self.write_fake_signor_pf_file(pfile)
            res = dloader.get_proteinfamily_map()
            self.assertEqual(['P27361', 'P28482'], res['ERK1/2'])
            self.assertEqual(['P27361', 'P28482'], res['SIGNOR-PF1'])
            self.assertEqual(['Q9HBW0', 'Q9UBY5', 'Q92633'], res['LPAR'])
            self.assertEqual(['Q9HBW0', 'Q9UBY5', 'Q92633'], res['SIGNOR-PF2'])
        finally:
            shutil.rmtree(temp_dir)

    def test_get_complexes_map(self):
        temp_dir = tempfile.mkdtemp()
        try:
            dloader = SignorDownloader(None, temp_dir)
            pfile = dloader.get_complexes_file()
            self.write_fake_signor_complexes_file(pfile)
            res = dloader.get_complexes_map()
            self.assertEqual(['P23511', 'P25208', 'Q13952'], res['NFY'])
            self.assertEqual(['P23511', 'P25208', 'Q13952'], res['SIGNOR-C1'])

            self.assertEqual(['P68104', 'Q8TB45'], res['mTORC2'])
            self.assertEqual(['P68104', 'Q8TB45'], res['SIGNOR-C2'])

        finally:
            shutil.rmtree(temp_dir)

    def test_get_pathways_map(self):
        temp_dir = tempfile.mkdtemp()
        try:
            dloader = SignorDownloader(None, temp_dir)
            with open(dloader.get_pathway_list_file(), 'w') as f:
                f.write('SIGNOR-AC\tAdipogenesis\n')
                f.write('SIGNOR-AD\tAlzheimer Disease\n')
                f.write('SIGNOR-AML\tAcute Myeloid Leukemia\n')
                f.write('SIGNOR-AML-IDH/TET\tAML-IDH/TET\n')

            res = dloader.get_pathways_map()
            self.assertEqual('Adipogenesis', res['SIGNOR-AC'])
            self.assertEqual('Alzheimer Disease', res['SIGNOR-AD'])
            self.assertEqual('Acute Myeloid Leukemia', res['SIGNOR-AML'])
            self.assertEqual('AML-IDH/TET', res['SIGNOR-AML-IDHTET'])
        finally:
            shutil.rmtree(temp_dir)

    def test_get_download_url(self):
        dloader = SignorDownloader('hi', None)
        self.assertEqual('hi/' +
                         SignorDownloader.PATHWAYDATA_DOWNLOAD_SCRIPT +
                         'haha', dloader._get_download_url('haha'))

        self.assertEqual('hi/' +
                         SignorDownloader.PATHWAYDATA_DOWNLOAD_SCRIPT +
                         'haha&relations=only',
                         dloader._get_download_url('haha',
                                                   relationsonly=True))

    def test_get_fulldownload_url(self):
        dloader = SignorDownloader('hi', None)
        self.assertEqual('hi/' + SignorDownloader.GETDATA_SCRIPT +
                         'yo',
                         dloader._get_fulldownload_url('yo'))

    def test_download_entity_file_success(self):
        dloader = SignorDownloader('http://hi', None)
        temp_dir = tempfile.mkdtemp()
        try:
            with requests_mock.mock() as m:
                m.post('http://hi/' + SignorDownloader.DOWNLOAD_COMPLEXES,
                       status_code=200,
                       text='hehe')
                tfile = os.path.join(temp_dir, 'hi')
                dloader._download_entity_file('entitytype', tfile)
                self.assertEqual(True, os.path.isfile(tfile))
                with open(tfile, 'r') as f:
                    self.assertEqual('hehe', f.read())
        finally:
            shutil.rmtree(temp_dir)

    def test_download_entity_file_error(self):
        dloader = SignorDownloader('http://hi', None)
        with requests_mock.mock() as m:
            m.post('http://hi/' + SignorDownloader.DOWNLOAD_COMPLEXES,
                   status_code=500,
                   text='hehe')
            try:
                dloader._download_entity_file('entitytype', 'foo')
                self.fail('Expected NDExLoadSignorError')
            except NDExLoadSignorError as ne:
                self.assertEqual('Got status code of 500 from signor',
                                 str(ne))

    def test_download_pathways_list_success(self):
        temp_dir = tempfile.mkdtemp()
        dloader = SignorDownloader('http://hi', temp_dir)

        try:
            with requests_mock.mock() as m:
                m.get('http://hi/' + SignorDownloader.PATHWAYDATA_SCRIPT,
                       status_code=200,
                       text='hehe')
                tfile = dloader.get_pathway_list_file()
                dloader._download_pathways_list()
                self.assertEqual(True,
                                 os.path.isfile(tfile))
                with open(tfile, 'r') as f:
                    self.assertEqual('hehe', f.read())
        finally:
            shutil.rmtree(temp_dir)

    def test_download_pathways_list_error(self):
        dloader = SignorDownloader('http://hi', None)
        with requests_mock.mock() as m:
            m.get('http://hi/' + SignorDownloader.PATHWAYDATA_SCRIPT,
                   status_code=500,
                   text='hehe')
            try:
                dloader._download_pathways_list()
                self.fail('Expected NDExLoadSignorError')
            except NDExLoadSignorError as ne:
                self.assertEqual('Got status code of 500 from signor',
                                 str(ne))

    def test_download_file_where_file_exists(self):
        temp_dir = tempfile.mkdtemp()
        try:
            tfile = os.path.join(temp_dir, 'hi')
            open(tfile, 'a').close()
            dloader = SignorDownloader('http://hi', tfile)
            dloader._download_file('http://hi', tfile)
        finally:
            shutil.rmtree(temp_dir)
